@@ -1,4 +1,4 @@
-/* $Id: drivers.c 4081 2006-12-05 13:44:39Z esr $ */
+/* $Id: drivers.c 4651 2008-01-01 14:53:16Z esr $ */
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
@@ -14,17 +14,18 @@
 #include "gpsd.h"
 
 extern struct gps_type_t zodiac_binary;
+extern struct gps_type_t ubx_binary;
 
 ssize_t generic_get(struct gps_device_t *session)
 {
     return packet_get(session->gpsdata.gps_fd, &session->packet);
 }
 
-#if defined(NMEA_ENABLE) || defined(SIRF_ENABLE) || defined(EVERMORE_ENABLE)  || defined(ITALK_ENABLE) 
+#if defined(NMEA_ENABLE) || defined(SIRF_ENABLE) || defined(EVERMORE_ENABLE)  || defined(ITRAX_ENABLE)  || defined(NAVCOM_ENABLE)
 ssize_t pass_rtcm(struct gps_device_t *session, char *buf, size_t rtcmbytes)
 /* most GPSes take their RTCM corrections straight up */
 {
-    return write(session->gpsdata.gps_fd, buf, rtcmbytes);
+    return gpsd_write(session, buf, rtcmbytes);
 }
 #endif
 
@@ -55,7 +56,15 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 #else
 	return 0;
 #endif /* EVERMORE_ENABLE */
-    } else if (session->packet.type == GARMIN_PACKET) {
+    } else if (session->packet.type == NAVCOM_PACKET) {
+  gpsd_report(LOG_WARN, "Navcom packet seen when NMEA expected.\n");
+#ifdef NAVCOM_ENABLE
+	(void)gpsd_switch_driver(session, "Navcom binary");
+	return navcom_parse(session, session->packet.outbuffer, session->packet.outbuflen);
+#else
+	return 0;
+#endif /* NAVCOM_ENABLE */
+} else if (session->packet.type == GARMIN_PACKET) {
 	gpsd_report(LOG_WARN, "Garmin packet seen when NMEA expected.\n");
 #ifdef GARMIN_ENABLE
 	/* we might never see a trigger, have this as a backstop */
@@ -66,6 +75,15 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 #endif /* GARMIN_ENABLE */
     } else if (session->packet.type == NMEA_PACKET) {
 	gps_mask_t st = 0;
+#ifdef GARMINTXT_ENABLE
+	if (session->packet.outbuflen >= 56) {
+		if ((char) *session->packet.outbuffer == '@') {
+		/* Garmin Simple Text packet received; it starts with '@' is terminated with \r\n and has length 57 bytes */
+			(void)gpsd_switch_driver(session, "Garmin Simple Text");
+			return garmintxt_parse(session);
+		}
+	}
+#endif /* GARMINTXT_ENABLE */
 	gpsd_report(LOG_IO, "<= GPS: %s", session->packet.outbuffer);
 	if ((st=nmea_parse((char *)session->packet.outbuffer, session))==0) {
 #ifdef NON_NMEA_ENABLE
@@ -85,7 +103,7 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 	    gpsd_report(LOG_WARN, "unknown sentence: \"%s\"\n", session->packet.outbuffer);
 	}
 #ifdef NMEADISC
-	if (session->gpsdata.ldisc == 0) {
+	if (session->ldisc == 0) {
 	    uid_t old;
 	    int ldisc = NMEADISC;
 
@@ -102,7 +120,7 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 	    if (seteuid(0) == -1)
 		gpsd_report(LOG_WARN, "can't seteuid(0) - %s", strerror(errno));
 	    else
-		gpsd_report(LOG_WARN, "seteuid(0) to enable timestamping");
+		gpsd_report(LOG_WARN, "seteuid(0) to enable timestamping\n");
 	    if (ioctl(session->gpsdata.gps_fd, TIOCSTSTAMP, &tstamps) < 0)
 		gpsd_report(LOG_WARN, "can't set kernel timestamping: %s\n",
 		    strerror(errno));
@@ -115,10 +133,10 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 	    else
 		gpsd_report(LOG_WARN, "activated nmea discipline\n");
 	/* this is a flag that shows if we've tried the setup */
-	session->gpsdata.ldisc = NMEADISC;
+	session->ldisc = NMEADISC;
 
 	if (old){
-	    gpsd_report(LOG_WARN, "giving up euid 0");
+	    gpsd_report(LOG_WARN, "giving up euid 0\n");
 	    (void)seteuid(old);
 	}
 	gpsd_report(LOG_WARN, "running with effective user ID %d\n", geteuid());
@@ -129,12 +147,7 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 	if (session->context->enable_ntpshm &&
 	    (st & TIME_SET) != 0 &&
 	    (session->gpsdata.fix.time!=session->last_fixtime)) {
-	    /* this magic number is derived from observation */
-	    /* GPS-18/USB -> 0.100 */
-	    /* GPS-18/LVC at 19200 -> 0.125 */
-	    /* GPS-18/LVC at 4800 -> 0.525*/
-	    /* Rob Jensen reports 0.675 */
-	    (void)ntpshm_put(session, session->gpsdata.fix.time + 0.400);
+	    (void)ntpshm_put(session, session->gpsdata.fix.time);
 	    session->last_fixtime = session->gpsdata.fix.time;
 	}
 #endif /* NTPSHM_ENABLE */
@@ -149,7 +162,7 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
      * The reason for splitting these probes up by packet sequence
      * number, interleaving them with the first few packet receives,
      * is because many generic-NMEA devices get confused if you send
-     * too much at them in one go.  
+     * too much at them in one go.
      *
      * A fast response to an early probe will change drivers so the
      * later ones won't be sent at all.  Thus, for best overall
@@ -157,23 +170,23 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
      * soonest.
      *
      * Note: don't make the trigger strings identical to the probe,
-     * because some NMEA devices (notably SiRFs) will just echo 
+     * because some NMEA devices (notably SiRFs) will just echo
      * unknown strings right back at you. A useful dodge is to append
-     * a comma to the trigger, because that won't be in the response 
+     * a comma to the trigger, because that won't be in the response
      * unless there is actual following data.
      */
     switch (seq) {
 #ifdef SIRF_ENABLE
     case 0:
-	/* 
+	/*
 	 * We used to try to probe for SiRF by issuing "$PSRF105,1"
 	 * and expecting "$Ack Input105.".  But it turns out this
 	 * only works for SiRF-IIs; SiRF-I and SiRF-III don't respond.
 	 * Thus the only reliable probe is to try to flip the SiRF into
 	 * binary mode, cluing in the library to revert it on close.
 	 */
-	(void)nmea_send(session->gpsdata.gps_fd, 
-			"$PSRF100,0,%d,%d,%d,0", 
+	(void)nmea_send(session->gpsdata.gps_fd,
+			"$PSRF100,0,%d,%d,%d,0",
 			session->gpsdata.baudrate,
 			9-session->gpsdata.stopbits,
 			session->gpsdata.stopbits);
@@ -194,7 +207,7 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
     case 3:
 	/* Enable checksum and GGA(1s), GLL(0s), GSA(1s), GSV(1s), RMC(1s), VTG(0s), PEMT101(1s) */
 	/* EverMore will reply with: \x10\x02\x04\x38\x8E\xC6\x10\x03 */
-	(void)gpsd_write(session, 
+	(void)gpsd_write(session,
 	    "\x10\x02\x12\x8E\x7F\x01\x01\x00\x01\x01\x01\x00\x01\x00\x00\x00\x00\x00\x00\x13\x10\x03", 22);
 	break;
 #endif /* EVERMORE_ENABLE */
@@ -204,6 +217,18 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
 	(void)nmea_send(session->gpsdata.gps_fd, "$PFST");
 	break;
 #endif /* ITRAX_ENABLE */
+#ifdef GPSCLOCK_ENABLE
+    case 5:
+	/* probe for Furuno Electric GH-79L4-N (GPSClock) */
+	(void)nmea_send(session->gpsdata.gps_fd, "$PFEC,GPsrq");
+	break;
+#endif /* GPSCLOCK_ENABLE */
+#ifdef ASHTECH_ENABLE
+    case 6:
+	/* probe for Ashtech -- expect $PASHR */
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHQ,RID");
+	break;
+#endif /* ASHTECH_ENABLE */
     default:
 	break;
     }
@@ -211,15 +236,15 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
 
 static struct gps_type_t nmea = {
     .typename       = "Generic NMEA",	/* full name of type */
-    .trigger        = NULL,		/* it's the default */
+    .trigger	    = NULL,		/* it's the default */
     .channels       = 12,		/* consumer-grade GPS */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
     .probe_detect   = NULL,		/* no probe */
-    .probe_subtype  = nmea_probe_subtype,	/* probe for special types */
+    .probe_subtype  = nmea_probe_subtype, /* probe for special types */
 #ifdef ALLOW_RECONFIGURE
     .configurator   = NULL,		/* enable what we need */
 #endif /* ALLOW_RECONFIGURE */
-    .get_packet     = generic_get,		/* use generic packet getter */
+    .get_packet     = generic_get,	/* use generic packet getter */
     .parse_packet   = nmea_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = pass_rtcm,	/* write RTCM data straight */
     .speed_switcher = NULL,		/* no speed switcher */
@@ -227,10 +252,10 @@ static struct gps_type_t nmea = {
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .cycle_chars    = -1,		/* not relevant, no rate switch */
 #ifdef ALLOW_RECONFIGURE
-    .revert         = NULL,		/* no setting-reversion method */
+    .revert	    = NULL,		/* no setting-reversion method */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = NULL,		/* no wrapup */
-    .cycle          = 1,		/* updates every second */
+    .wrapup	    = NULL,		/* no wrapup */
+    .cycle	    = 1,		/* updates every second */
 };
 
 #ifdef GARMIN_ENABLE
@@ -245,7 +270,8 @@ static void garmin_nmea_configurator(struct gps_device_t *session, unsigned int
 {
 #if defined(NMEA_ENABLE)
     /*
-     * Receivers like the Garmin GPS-10 don't handle having a lot of 
+     * Receivers like the Garmin GPS-10 don't handle having having a lot of
+     * probes shoved at them very well.
      */
     switch (seq) {
     case 0:
@@ -285,11 +311,11 @@ static void garmin_nmea_configurator(struct gps_device_t *session, unsigned int
 
 static struct gps_type_t garmin = {
     .typename       = "Garmin Serial",	/* full name of type */
-    .trigger        = "$PGRMC,",	/* Garmin private */
+    .trigger	    = "$PGRMC,",	/* Garmin private */
     .channels       = 12,		/* not used by this driver */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
     .probe_detect   = NULL,		/* no probe */
-    .probe_subtype   = NULL,		/* no further querying */
+    .probe_subtype  = NULL,		/* no further querying */
 #ifdef ALLOW_RECONFIGURE
     .configurator   = garmin_nmea_configurator,/* enable what we need */
 #endif /*ALLOW_RECONFIGURE */
@@ -301,13 +327,74 @@ static struct gps_type_t garmin = {
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .cycle_chars    = -1,		/* not relevant, no rate switch */
 #ifdef ALLOW_RECONFIGURE
-    .revert         = NULL,		/* no setting-reversion method */
+    .revert	    = NULL,		/* no setting-reversion method */
 #endif /*ALLOW_RECONFIGURE */
-    .wrapup         = NULL,		/* no wrapup */
-    .cycle          = 1,		/* updates every second */
+    .wrapup	    = NULL,		/* no wrapup */
+    .cycle	    = 1,		/* updates every second */
 };
 #endif /* GARMIN_ENABLE */
 
+#ifdef ASHTECH_ENABLE
+/**************************************************************************
+ *
+ * Ashtech (then Thales, now Magellan Professional) Receivers
+ *
+ **************************************************************************/
+
+#ifdef ALLOW_RECONFIGURE
+static void ashtech_configure(struct gps_device_t *session, unsigned int seq)
+{
+    if (seq == 0){
+	/* turn WAAS on. can't hurt... */
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,WAS,ON");
+	/* reset to known output state */
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,ALL,A,OFF");
+	/* then turn on some useful sentences */
+#ifdef ASHTECH_NOTYET
+	/* we could parse these, but they're oversize so they get dropped */
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,POS,A,ON");
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,SAT,A,ON");
+#else
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,GGA,A,ON");
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,GSA,A,ON");
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,GSV,A,ON");
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,RMC,A,ON");
+#endif
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,ZDA,A,ON");
+    }
+}
+#endif /* ALLOW_RECONFIGURE */
+
+static void ashtech_ping(struct gps_device_t *session)
+{
+	(void)nmea_send(session->gpsdata.gps_fd, "$PASHQ,RID");
+}
+
+static struct gps_type_t ashtech = {
+    .typename       = "Ashtech",	/* full name of type */
+    .trigger	    = "$PASHR,RID,",	/* Ashtech receivers respond thus */
+    .channels       = 24,		/* not used, GG24 has 24 channels */
+    .probe_wakeup   = ashtech_ping,	/* no wakeup to be done before hunt */
+    .probe_detect   = NULL,		/* no probe */
+    .probe_subtype  = NULL,		/* to be sent unconditionally */
+#ifdef ALLOW_RECONFIGURE
+    .configurator   = ashtech_configure, /* change its sentence set */
+#endif /* ALLOW_RECONFIGURE */
+    .get_packet     = generic_get,	/* how to get a packet */
+    .parse_packet   = nmea_parse_input,	/* how to interpret a packet */
+    .rtcm_writer    = pass_rtcm,	/* write RTCM data straight */
+    .speed_switcher = NULL,		/* no speed switcher */
+    .mode_switcher  = NULL,		/* no mode switcher */
+    .rate_switcher  = NULL,		/* no sample-rate switcher */
+    .cycle_chars    = -1,		/* not relevant, no rate switch */
+#ifdef ALLOW_RECONFIGURE
+    .revert	    = NULL,		/* no setting-reversion method */
+#endif /* ALLOW_RECONFIGURE */
+    .wrapup	    = NULL,		/* no wrapup */
+    .cycle	    = 1,		/* updates every second */
+};
+#endif /* ASHTECH_ENABLE */
+
 #ifdef FV18_ENABLE
 /**************************************************************************
  *
@@ -330,10 +417,10 @@ static void fv18_configure(struct gps_device_t *session, unsigned int seq)
 
 static struct gps_type_t fv18 = {
     .typename       = "San Jose Navigation FV18",	/* full name of type */
-    .trigger        = "$PFEC,GPint,",	/* FV18s should echo the probe */
+    .trigger	    = "$PFEC,GPint,",	/* FV18s should echo the probe */
     .channels       = 12,		/* not used by this driver */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
-    .probe_detect   = NULL,		/* mo probe */
+    .probe_detect   = NULL,		/* no probe */
     .probe_subtype  = NULL,		/* to be sent unconditionally */
 #ifdef ALLOW_RECONFIGURE
     .configurator   = fv18_configure,	/* change its sentence set */
@@ -346,13 +433,61 @@ static struct gps_type_t fv18 = {
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .cycle_chars    = -1,		/* not relevant, no rate switch */
 #ifdef ALLOW_RECONFIGURE
-    .revert         = NULL,		/* no setting-reversion method */
+    .revert	    = NULL,		/* no setting-reversion method */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = NULL,		/* no wrapup */
-    .cycle          = 1,		/* updates every second */
+    .wrapup	    = NULL,		/* no wrapup */
+    .cycle	    = 1,		/* updates every second */
 };
 #endif /* FV18_ENABLE */
 
+#ifdef GPSCLOCK_ENABLE
+/**************************************************************************
+ *
+ * Furuno Electric GPSClock (GH-79L4)
+ *
+ **************************************************************************/
+
+/*
+ * Based on http://www.tecsys.de/fileadmin/user_upload/pdf/gh79_1an_intant.pdf
+ */
+
+static void gpsclock_probe_subtype(struct gps_device_t *session, unsigned int seq)
+{
+    /*
+     * Michael St. Laurent <mikes@hartwellcorp.com> reports that you have to
+     * ignore the trailing PPS edge when extracting time from this chip.
+     */
+    if (seq == 0) {
+	gpsd_report(LOG_INF, "PPS trailing edge will be ignored");
+	session->driver.nmea.ignore_trailing_edge = true;
+    }
+}
+
+static struct gps_type_t gpsclock = {
+    .typename       = "Furuno Electric GH-79L4",	/* full name of type */
+    .trigger	    = "$PFEC,GPssd",	/* GPSclock should echo the probe */
+    .channels       = 12,		/* not used by this driver */
+    .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
+    .probe_detect   = NULL,		/* no probe */
+    .probe_subtype  = gpsclock_probe_subtype, /* to be sent unconditionally */
+#ifdef ALLOW_RECONFIGURE
+    .configurator   = NULL,		/* change its sentence set */
+#endif /* ALLOW_RECONFIGURE */
+    .get_packet     = generic_get,	/* how to get a packet */
+    .parse_packet   = nmea_parse_input,	/* how to interpret a packet */
+    .rtcm_writer    = pass_rtcm,	/* write RTCM data straight */
+    .speed_switcher = NULL,		/* no speed switcher */
+    .mode_switcher  = NULL,		/* no mode switcher */
+    .rate_switcher  = NULL,		/* sample rate is fixed */
+    .cycle_chars    = -1,		/* sample rate is fixed */
+#ifdef ALLOW_RECONFIGURE
+    .revert	    = NULL,		/* no setting-reversion method */
+#endif /* ALLOW_RECONFIGURE */
+    .wrapup	    = NULL,		/* no wrapup */
+    .cycle	    = 1,		/* updates every second */
+};
+#endif /* GPSCLOCK_ENABLE */
+
 #ifdef TRIPMATE_ENABLE
 /**************************************************************************
  *
@@ -402,10 +537,10 @@ static struct gps_type_t tripmate = {
     .rate_switcher = NULL,			/* no sample-rate switcher */
     .cycle_chars   = -1,			/* no rate switch */
 #ifdef ALLOW_RECONFIGURE
-    .revert         = NULL,		/* no setting-reversion method */
+    .revert	   = NULL,			/* no setting-reversion method */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = NULL,			/* no wrapup */
-    .cycle          = 1,			/* updates every second */
+    .wrapup	   = NULL,			/* no wrapup */
+    .cycle	   = 1,				/* updates every second */
 };
 #endif /* TRIPMATE_ENABLE */
 
@@ -433,7 +568,7 @@ static void earthmate_close(struct gps_device_t *session)
 static void earthmate_probe_subtype(struct gps_device_t *session, unsigned int seq)
 {
     if (seq == 0) {
-	(void)write(session->gpsdata.gps_fd, "EARTHA\r\n", 8);
+	(void)gpsd_write(session, "EARTHA\r\n", 8);
 	(void)usleep(10000);
 	/*@i@*/session->device_type = &zodiac_binary;
 	zodiac_binary.wrapup = earthmate_close;
@@ -446,7 +581,7 @@ static struct gps_type_t earthmate = {
     .typename      = "Delorme EarthMate (pre-2003, Zodiac chipset)",
     .trigger       = "EARTHA",			/* Earthmate trigger string */
     .channels      = 12,			/* not used by NMEA parser */
-    .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
+    .probe_wakeup  = NULL,			/* no wakeup to be done before hunt */
     .probe_detect  = NULL,			/* no probe */
     .probe_subtype = earthmate_probe_subtype,	/* switch us to Zodiac mode */
 #ifdef ALLOW_RECONFIGURE
@@ -460,153 +595,14 @@ static struct gps_type_t earthmate = {
     .rate_switcher = NULL,			/* no sample-rate switcher */
     .cycle_chars   = -1,			/* no rate switch */
 #ifdef ALLOW_RECONFIGURE
-    .revert         = NULL,		/* no setting-reversion method */
+    .revert	   = NULL,			/* no setting-reversion method */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = NULL,			/* no wrapup code */
-    .cycle          = 1,			/* updates every second */
+    .wrapup	   = NULL,			/* no wrapup code */
+    .cycle	   = 1,				/* updates every second */
 };
 /*@ -redef @*/
 #endif /* EARTHMATE_ENABLE */
 
-
-#ifdef ITRAX_ENABLE
-/**************************************************************************
- *
- * The NMEA mode of the iTrax chipset, as used in the FastTrax and others.
- *
- * As described by v1.31 of the NMEA Protocol Specification for the
- * iTrax02 Evaluation Kit, 2003-06-12.
- * v1.18 of the  manual, 2002-19-6, describes effectively
- * the same protocol, but without ZDA.
- *
- **************************************************************************/
-
-/*
- * Enable GGA=0x2000, RMC=0x8000, GSA=0x0002, GSV=0x0001, ZDA=0x0004.
- * Disable GLL=0x1000, VTG=0x4000, FOM=0x0020, PPS=0x0010.
- * This is 82+75+67+(3*60)+34 = 438 characters 
- * 
- * 1200   => at most 1 fix per 4 seconds
- * 2400   => at most 1 fix per 2 seconds
- * 4800   => at most 1 fix per 1 seconds
- * 9600   => at most 2 fixes per second
- * 19200  => at most 4 fixes per second
- * 57600  => at most 13 fixes per second
- * 115200 => at most 26 fixes per second
- *
- * We'd use FOM, but they don't specify a confidence interval.
- */
-#define ITRAX_MODESTRING	"$PFST,NMEA,A007,%d\r\n"
-
-static int literal_send(int fd, const char *fmt, ... )
-/* ship a raw command to the GPS */
-{
-    int status;
-    char buf[BUFSIZ];
-    va_list ap;
-
-    va_start(ap, fmt) ;
-    (void)vsnprintf(buf, sizeof(buf), fmt, ap);
-    va_end(ap);
-    status = (int)write(fd, buf, strlen(buf));
-    if (status == (int)strlen(buf)) {
-	gpsd_report(LOG_IO, "=> GPS: %s\n", buf);
-	return status;
-    } else {
-	gpsd_report(LOG_WARN, "=> GPS: %s FAILED\n", buf);
-	return -1;
-    }
-}
-
-static void itrax_probe_subtype(struct gps_device_t *session, unsigned int seq)
-/* start it reporting */
-{
-    if (seq == 0) {
-	/* initialize GPS clock with current system time */ 
-	struct tm when;
-	double integral, fractional;
-	time_t intfixtime;
-	char buf[31], frac[6];
-	fractional = modf(timestamp(), &integral);
-	intfixtime = (time_t)integral;
-	(void)gmtime_r(&intfixtime, &when);
-	/* FIXME: so what if my local clock is wrong? */
-	(void)strftime(buf, sizeof(buf), "$PFST,INITAID,%H%M%S.XX,%d%m%y\r\n", &when);
-	(void)snprintf(frac, sizeof(frac), "%.2f", fractional);
-	buf[21] = frac[2]; buf[22] = frac[3];
-	(void)literal_send(session->gpsdata.gps_fd, buf);
-	/* maybe this should be considered a reconfiguration? */
-	(void)literal_send(session->gpsdata.gps_fd, "$PFST,START\r\n");
-    }
-}
-
-#ifdef ALLOW_RECONFIGURE
-static void itrax_configurator(struct gps_device_t *session, int seq)
-/* set synchronous mode */
-{
-    if (seq == 0) {
-	(void)literal_send(session->gpsdata.gps_fd, "$PFST,SYNCMODE,1\r\n");
-	(void)literal_send(session->gpsdata.gps_fd, 
-		    ITRAX_MODESTRING, session->gpsdata.baudrate);
-    }
-}
-#endif /* ALLOW_RECONFIGURE */
-
-static bool itrax_speed(struct gps_device_t *session, speed_t speed)
-/* change the baud rate */
-{
-#ifdef ALLOW_RECONFIGURE
-    return literal_send(session->gpsdata.gps_fd, ITRAX_MODESTRING, speed) >= 0;
-#else
-    return false;
-#endif /* ALLOW_RECONFIGURE */
-}
-
-static bool itrax_rate(struct gps_device_t *session, double rate)
-/* change the sample rate of the GPS */
-{
-#ifdef ALLOW_RECONFIGURE
-    return literal_send(session->gpsdata.gps_fd, "$PSFT,FIXRATE,%d\r\n", rate) >= 0;
-#else
-    return false;
-#endif /* ALLOW_RECONFIGURE */
-}
-
-static void itrax_wrap(struct gps_device_t *session)
-/* stop navigation, this cuts the power drain */
-{
-#ifdef ALLOW_RECONFIGURE
-    (void)literal_send(session->gpsdata.gps_fd, "$PFST,SYNCMODE,0\r\n");
-#endif /* ALLOW_RECONFIGURE */
-    (void)literal_send(session->gpsdata.gps_fd, "$PFST,STOP\r\n");
-}
-
-/*@ -redef @*/
-static struct gps_type_t itrax = {
-    .typename      = "iTrax",		/* full name of type */
-    .trigger       = "$PFST,OK",	/* tells us to switch to Itrax */
-    .channels      = 12,		/* consumer-grade GPS */
-    .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
-    .probe_detect  = NULL,		/* no probe */
-    .probe_subtype = itrax_probe_subtype,	/* initialize */
-#ifdef ALLOW_RECONFIGURE
-    .configurator  = itrax_configurator,/* set synchronous mode */
-#endif /* ALLOW_RECONFIGURE */
-    .get_packet    = generic_get,	/* how to get a packet */
-    .parse_packet  = nmea_parse_input,	/* how to interpret a packet */
-    .rtcm_writer   = NULL,		/* iTrax doesn't support DGPS/WAAS/EGNOS */
-    .speed_switcher= itrax_speed,	/* no speed switcher */
-    .mode_switcher = NULL,		/* no mode switcher */
-    .rate_switcher = itrax_rate,	/* there's a sample-rate switcher */
-    .cycle_chars   = 438,		/* not relevant, no rate switch */
-#ifdef ALLOW_RECONFIGURE
-    .revert         = NULL,		/* no setting-reversion method */
-#endif /* ALLOW_RECONFIGURE */
-    .wrapup         = itrax_wrap,	/* sleep the receiver */
-    .cycle          = 1,		/* updates every second */
-};
-/*@ -redef @*/
-#endif /* ITRAX_ENABLE */
 #endif /* NMEA_ENABLE */
 
 #ifdef TNT_ENABLE
@@ -635,7 +631,7 @@ static void tnt_add_checksum(char *sentence)
     if (*p == '@') {
 	p++;
     } else {
-        gpsd_report(LOG_ERROR, "Bad TNT sentence: '%s'\n", sentence);
+	gpsd_report(LOG_ERROR, "Bad TNT sentence: '%s'\n", sentence);
     }
     while ( ((c = *p) != '*') && (c != '\0')) {
 	sum ^= c;
@@ -671,31 +667,31 @@ static int tnt_send(int fd, const char *fmt, ... )
 /*
  * The True North compass won't start talking
  * unless you ask it to. So to identify it we
- * need to query for it's ID string.
+ * need to query for its ID string.
  */
 static int tnt_packet_sniff(struct gps_device_t *session)
 {
     unsigned int n, count = 0;
 
     gpsd_report(LOG_RAW, "tnt_packet_sniff begins\n");
-    for (n = 0; n < TNT_SNIFF_RETRIES; n++) 
+    for (n = 0; n < TNT_SNIFF_RETRIES; n++)
     {
       count = 0;
       (void)tnt_send(session->gpsdata.gps_fd, "@X?");
       if (ioctl(session->gpsdata.gps_fd, FIONREAD, &count) < 0)
-          return BAD_PACKET;
+	  return BAD_PACKET;
       if (count == 0) {
-          //int delay = 10000000000.0 / session->gpsdata.baudrate;
-          //gpsd_report(LOG_RAW, "usleep(%d)\n", delay);
-          //usleep(delay);
-          gpsd_report(LOG_RAW, "sleep(1)\n");
-          (void)sleep(1);
+	  //int delay = 10000000000.0 / session->gpsdata.baudrate;
+	  //gpsd_report(LOG_RAW, "usleep(%d)\n", delay);
+	  //usleep(delay);
+	  gpsd_report(LOG_RAW, "sleep(1)\n");
+	  (void)sleep(1);
       } else if (generic_get(session) >= 0) {
-        if((session->packet.type == NMEA_PACKET)&&(session->packet.state == NMEA_RECOGNIZED))
-        {
-          gpsd_report(LOG_RAW, "tnt_packet_sniff returns %d\n",session->packet.type);
-          return session->packet.type;
-        }
+	if((session->packet.type == NMEA_PACKET)&&(session->packet.state == NMEA_RECOGNIZED))
+	{
+	  gpsd_report(LOG_RAW, "tnt_packet_sniff returns %d\n",session->packet.type);
+	  return session->packet.type;
+	}
       }
     }
 
@@ -746,17 +742,17 @@ static bool tnt_probe(struct gps_device_t *session)
   for (ip = rates; ip < rates + sizeof(rates)/sizeof(rates[0]); ip++)
       if (ip == rates || *ip != rates[0])
       {
-          gpsd_report(LOG_PROG, "hunting at speed %d\n", *ip);
-          gpsd_set_speed(session, *ip, 'N',1);
-          if (tnt_packet_sniff(session) != BAD_PACKET)
-              return true;
+	  gpsd_report(LOG_PROG, "hunting at speed %d\n", *ip);
+	  gpsd_set_speed(session, *ip, 'N',1);
+	  if (tnt_packet_sniff(session) != BAD_PACKET)
+	      return true;
       }
   return false;
 }
 
 struct gps_type_t trueNorth = {
     .typename       = "True North",	/* full name of type */
-    .trigger        = " TNT1500",
+    .trigger	    = " TNT1500",
     .channels       = 0,		/* not an actual GPS at all */
     .probe_wakeup   = NULL,		/* this will become a real method */
     .probe_detect   = tnt_probe,	/* probe by sending ID query */
@@ -766,16 +762,16 @@ struct gps_type_t trueNorth = {
 #endif /* ALLOW_RECONFIGURE */
     .get_packet     = generic_get,	/* how to get a packet */
     .parse_packet   = nmea_parse_input,	/* how to interpret a packet */
-    .rtcm_writer    = NULL,	        /* Don't send */
+    .rtcm_writer    = NULL,		/* Don't send */
     .speed_switcher = NULL,		/* no speed switcher */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no wrapup */
     .cycle_chars    = -1,		/* not relevant, no rate switch */
 #ifdef ALLOW_RECONFIGURE
-    .revert         = NULL,		/* no setting-reversion method */
+    .revert	    = NULL,		/* no setting-reversion method */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = NULL,		/* no wrapup */
-    .cycle          = 20,		/* updates per second */
+    .wrapup	    = NULL,		/* no wrapup */
+    .cycle	    = 20,		/* updates per second */
 };
 #endif
 #ifdef RTCM104_ENABLE
@@ -788,7 +784,7 @@ struct gps_type_t trueNorth = {
 static gps_mask_t rtcm104_analyze(struct gps_device_t *session)
 {
     rtcm_unpack(&session->gpsdata.rtcm, (char *)session->packet.isgps.buf);
-    gpsd_report(LOG_RAW, "RTCM packet type 0x%02x length %d words: %s\n", 
+    gpsd_report(LOG_RAW, "RTCM packet type 0x%02x length %d words: %s\n",
 		session->gpsdata.rtcm.type,
 		session->gpsdata.rtcm.length+2,
 		gpsd_hexdump(session->packet.isgps.buf, (session->gpsdata.rtcm.length+2)*sizeof(isgps30bits_t)));
@@ -813,25 +809,71 @@ static struct gps_type_t rtcm104 = {
     .rate_switcher = NULL,		/* no sample-rate switcher */
     .cycle_chars   = -1,		/* not relevant, no rate switch */
 #ifdef ALLOW_RECONFIGURE
-    .revert         = NULL,		/* no setting-reversion method */
+    .revert	   = NULL,		/* no setting-reversion method */
 #endif /* ALLOW_RECONFIGURE */
-    .wrapup         = NULL,		/* no wrapup code */
-    .cycle          = 1,		/* updates every second */
+    .wrapup	   = NULL,		/* no wrapup code */
+    .cycle	   = 1,			/* updates every second */
 };
 #endif /* RTCM104_ENABLE */
 
+#ifdef GARMINTXT_ENABLE
+/**************************************************************************
+ *
+ * Garmin Simple Text protocol
+ *
+ **************************************************************************/
+
+static gps_mask_t garmintxt_parse_input(struct gps_device_t *session)
+{
+    //gpsd_report(LOG_PROG, "Garmin Simple Text packet\n");
+    return garmintxt_parse(session);
+}
+
+
+static struct gps_type_t garmintxt = {
+    .typename      = "Garmin Simple Text",		/* full name of type */
+    .trigger       = NULL,		/* no recognition string */
+    .channels      = 0,			/* not used */
+    .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
+    .probe_detect  = NULL,		/* no probe */
+    .probe_subtype = NULL,		/* no subtypes */
+#ifdef ALLOW_RECONFIGURE
+    .configurator  = NULL,		/* no configurator */
+#endif /* ALLOW_RECONFIGURE */
+    .get_packet    = generic_get,	/* how to get a packet */
+    .parse_packet  = garmintxt_parse_input,	/*  */
+    .rtcm_writer   = NULL,		/* don't send RTCM data,  */
+    .speed_switcher= NULL,		/* no speed switcher */
+    .mode_switcher = NULL,		/* no mode switcher */
+    .rate_switcher = NULL,		/* no sample-rate switcher */
+    .cycle_chars   = -1,		/* not relevant, no rate switch */
+#ifdef ALLOW_RECONFIGURE
+    .revert	   = NULL,		/* no setting-reversion method */
+#endif /* ALLOW_RECONFIGURE */
+    .wrapup	   = NULL,		/* no wrapup code */
+    .cycle	   = 1,			/* updates every second */
+};
+#endif /* GARMINTXT_ENABLE */
+
 extern struct gps_type_t garmin_usb_binary, garmin_ser_binary;
 extern struct gps_type_t sirf_binary, tsip_binary;
 extern struct gps_type_t evermore_binary, italk_binary;
+extern struct gps_type_t navcom_binary;
 
 /*@ -nullassign @*/
 /* the point of this rigamarole is to not have to export a table size */
 static struct gps_type_t *gpsd_driver_array[] = {
 #ifdef NMEA_ENABLE
-    &nmea, 
+    &nmea,
+#ifdef ASHTECH_ENABLE
+    &ashtech,
+#endif /* ASHTECHV18_ENABLE */
 #ifdef FV18_ENABLE
     &fv18,
 #endif /* FV18_ENABLE */
+#ifdef GPSCLOCK_ENABLE
+    &gpsclock,
+#endif /* GPSCLOCK_ENABLE */
 #ifdef GARMIN_ENABLE
     &garmin,
 #endif /* GARMIN_ENABLE */
@@ -839,37 +881,43 @@ static struct gps_type_t *gpsd_driver_array[] = {
     &tripmate,
 #endif /* TRIPMATE_ENABLE */
 #ifdef EARTHMATE_ENABLE
-    &earthmate, 
+    &earthmate,
 #endif /* EARTHMATE_ENABLE */
-#ifdef ITRAX_ENABLE
-    &itrax, 
-#endif /* ITRAX_ENABLE */
 #endif /* NMEA_ENABLE */
 #ifdef ZODIAC_ENABLE
     &zodiac_binary,
 #endif /* ZODIAC_ENABLE */
+#ifdef NAVCOM_ENABLE
+    &navcom_binary,
+#endif /* NAVCOM_ENABLE */
+#ifdef UBX_ENABLE
+    &ubx_binary,
+#endif /* UBX_ENABLE */
 #ifdef GARMIN_ENABLE
     &garmin_usb_binary,
     &garmin_ser_binary,
 #endif /* GARMIN_ENABLE */
 #ifdef SIRF_ENABLE
-    &sirf_binary, 
+    &sirf_binary,
 #endif /* SIRF_ENABLE */
 #ifdef TSIP_ENABLE
-    &tsip_binary, 
+    &tsip_binary,
 #endif /* TSIP_ENABLE */
 #ifdef TNT_ENABLE
     &trueNorth,
 #endif /* TSIP_ENABLE */
 #ifdef EVERMORE_ENABLE
-    &evermore_binary, 
+    &evermore_binary,
 #endif /* EVERMORE_ENABLE */
-#ifdef ITALK_ENABLE
-    &italk_binary, 
-#endif /* ITALK_ENABLE */
+#ifdef ITRAX_ENABLE
+    &italk_binary,
+#endif /* ITRAX_ENABLE */
 #ifdef RTCM104_ENABLE
-    &rtcm104, 
+    &rtcm104,
 #endif /* RTCM104_ENABLE */
+#ifdef GARMINTXT_ENABLE
+    &garmintxt,
+#endif /* GARMINTXT_ENABLE */
     NULL,
 };
 /*@ +nullassign @*/
