@@ -1,4 +1,4 @@
-/* $Id: evermore.c 4116 2006-12-11 05:30:09Z esr $ */
+/* $Id: evermore.c 4373 2007-06-01 22:30:43Z esr $ */
 /*
  * This is the gpsd driver for EverMore GPSes operating in binary mode.
  * About the only thing this gives us that NMEA won't is TDOP.
@@ -172,7 +172,7 @@ static bool evermore_write(struct gps_device_t *session, unsigned char *msg, siz
    gpsd_report(LOG_IO, "writing EverMore control type 0x%02x: %s\n", msg[0], 
 	       gpsd_hexdump(stuffed, len));
 #ifdef ALLOW_RECONFIGURE
-   return gpsd_write(session, stuffed, len);
+   return (gpsd_write(session, stuffed, len) == (ssize_t)len);
 #else
    return 0;
 #endif /* ALLOW_RECONFIGURE */
