@@ -1,4 +1,4 @@
-/* $Id: garmin.c 4117 2006-12-11 05:36:49Z garyemiller $ */
+/* $Id: garmin.c 4341 2007-04-06 21:14:02Z ckuethe $ */
 /*
  * Handle the Garmin binary packet format supported by the USB Garmins
  * tested with the Garmin 18 and other models.  This driver is NOT for
@@ -327,7 +327,23 @@ gps_mask_t PrintSERPacket(struct gps_device_t *session, unsigned char pkt_id
 	gpsd_report(LOG_PROG, "time_l: %ld\n", (long int)time_l);
 
 	session->gpsdata.fix.latitude = radtodeg(pvt->lat);
+	/* sanity check the lat */
+	if ( 90.0 < session->gpsdata.fix.latitude ) {
+		session->gpsdata.fix.latitude = 90.0;
+		gpsd_report(LOG_INF, "ERROR: Latitude overrange\n");
+	} else if ( -90.0 > session->gpsdata.fix.latitude ) {
+		session->gpsdata.fix.latitude = -90.0;
+		gpsd_report(LOG_INF, "ERROR: Latitude negative overrange\n");
+	}
 	session->gpsdata.fix.longitude = radtodeg(pvt->lon);
+	/* sanity check the lon */
+	if ( 180.0 < session->gpsdata.fix.longitude ) {
+		session->gpsdata.fix.longitude = 180.0;
+		gpsd_report(LOG_INF, "ERROR: Longitude overrange\n");
+	} else if ( -180.0 > session->gpsdata.fix.longitude ) {
+		session->gpsdata.fix.longitude = -180.0;
+		gpsd_report(LOG_INF, "ERROR: Longitude negative overrange\n");
+	}
 
 	// altitude over WGS84 converted to MSL
 	session->gpsdata.fix.altitude = pvt->alt + pvt->msl_hght;
@@ -627,22 +643,21 @@ static void Build_Send_USB_Packet( struct gps_device_t *session,
 #endif
         (void)PrintUSBPacket ( session,  thePacket);
 
-	theBytesReturned = write( session->gpsdata.gps_fd
-				  , thePacket, (size_t)theBytesToWrite);
+	theBytesReturned = gpsd_write( session , thePacket,
+	    (size_t)theBytesToWrite);
 	gpsd_report(LOG_IO, "SendPacket(), wrote %d bytes\n", theBytesReturned);
 
 	// Garmin says:
 	// If the packet size was an exact multiple of the USB packet
 	// size, we must make a final write call with no data
 
-	// as a practical matter no known pckets are 64 bytes long so
+	// as a practical matter no known packets are 64 bytes long so
         // this is untested
 
 	// So here goes just in case
 	if( 0 == (theBytesToWrite % ASYNC_DATA_SIZE) ) {
 		char *n = "";
-		theBytesReturned = write( session->gpsdata.gps_fd
-		    , &n, 0);
+		theBytesReturned = gpsd_write( session , &n, 0);
 	}
 }
 /* build and send a packet in serial protocol */
@@ -698,8 +713,8 @@ static void Build_Send_SER_Packet( struct gps_device_t *session,
 			       (int)buffer0[2], 
 			       (unsigned char *)(buffer0 + 3));
 
-	theBytesReturned = write( session->gpsdata.gps_fd
-				  , thePacket, (size_t)theBytesToWrite);
+	theBytesReturned = gpsd_write( session, thePacket,
+	    (size_t)theBytesToWrite);
 	gpsd_report(LOG_IO, "SendPacket(), wrote %d bytes\n", theBytesReturned);
 
 }
@@ -953,7 +968,7 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
 
     // sending ACK too soon might hang the session
     // so send ACK last, after a pause
-    usleep(300);
+    (void)usleep(300);
     Send_ACK();
     /*@ +usedef +compdef @*/
     return mask;
@@ -978,8 +993,7 @@ static void garmin_switcher(struct gps_device_t *session, int mode)
 #ifdef ALLOW_RECONFIGURE
     if (mode == 0) {
 	const char *switcher = "\x10\x0A\x02\x26\x00\xCE\x10\x03";
-	int status = (int)write(session->gpsdata.gps_fd, 
-				switcher, strlen(switcher));
+	int status = (int)gpsd_write(session, switcher, strlen(switcher));
 	if (status == (int)strlen(switcher)) {
 	    gpsd_report(LOG_IO, "=> GPS: turn off binary %02x %02x %02x... \n"
 			, switcher[0], switcher[1], switcher[2]);
