@@ -1,8 +1,9 @@
-/* $Id: gpsd.c 4136 2006-12-14 21:20:57Z ckuethe $ */
+/* $Id: gpsd.c 4640 2007-12-27 18:53:52Z esr $ */
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <syslog.h>
@@ -37,6 +38,9 @@
 #if defined(HAVE_SYS_TIME_H)
 #include <sys/time.h>
 #endif
+#ifdef HAVE_SETLOCALE
+#include <locale.h>
+#endif
 
 #ifdef DBUS_ENABLE
 #include <gpsd_dbus.h>
@@ -46,14 +50,29 @@
 #include "timebase.h"
 
 /*
- * Timeout policy.  We can't rely on clients closing connections 
- * correctly, so we need timeouts to tell us when it's OK to 
+ * The name of a tty device from which to pick up whatever the local
+ * owning group for tty devices is.  Used when we drop privileges.
+ */
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#define PROTO_TTY "/dev/tty00"  /* correct for *BSD */
+#else
+#define PROTO_TTY "/dev/ttyS0"	/* correct for Linux */
+#endif
+
+/* Name of (unprivileged) user to change to when we drop privileges. */
+#ifndef GPSD_USER
+#define GPSD_USER	"nobody"
+#endif
+
+/*
+ * Timeout policy.  We can't rely on clients closing connections
+ * correctly, so we need timeouts to tell us when it's OK to
  * reclaim client fds.  The assignment timeout fends off programs
  * that open connections and just sit there, not issuing a W or
  * doing anything else that triggers a device assignment.  Clients
- * in watcher or raw mode that don't read their data will get dropped 
- * when throttled_write() fills up the outbound buffers and the 
- * NOREAD_TIMEOUT expires.  Clients in the original polling mode have 
+ * in watcher or raw mode that don't read their data will get dropped
+ * when throttled_write() fills up the outbound buffers and the
+ * NOREAD_TIMEOUT expires.  Clients in the original polling mode have
  * to be timed out as well.
  */
 #define ASSIGNMENT_TIMEOUT	60
@@ -62,12 +81,6 @@
 
 #define QLEN			5
 
-/*
- * The name of a tty device from which to pick up whatever the local
- * owning group for tty devices is.  Used when we drop privileges.
- */
-#define PROTO_TTY "/dev/ttyS0"
-
 #define sub_index(s) (s - subscribers)
 
 static fd_set all_fds;
@@ -78,33 +91,35 @@ static bool nowait = false;
 static jmp_buf restartbuf;
 /*@ -initallelements -nullassign -nullderef @*/
 static struct gps_context_t context = {
-    .valid              = 0, 
-    .sentdgps           = false, 
-    .dgnss_service      = dgnss_none,
-    .fixcnt             = 0, 
-    .dsock              = -1, 
-    .dgnss_privdata     = NULL,
-    .rtcmbytes          = 0, 
-    .rtcmbuf            = {'\0'}, 
-    .rtcmtime           = 0,
-    .leap_seconds       = LEAP_SECONDS, 
-    .century            = CENTURY_BASE, 
+    .valid	    = 0,
+    .readonly	    = false,
+    .sentdgps	    = false,
+    .dgnss_service  = dgnss_none,
+    .fixcnt	    = 0,
+    .dsock	    = -1,
+    .dgnss_privdata = NULL,
+    .rtcmbytes	    = 0,
+    .rtcmbuf	    = {'\0'},
+    .rtcmtime	    = 0,
+    .leap_seconds   = LEAP_SECONDS,
+    .century	    = CENTURY_BASE,
 #ifdef NTPSHM_ENABLE
-    .enable_ntpshm      = false,
-    .shmTime            = {0},
-    .shmTimeInuse       = {0},
+    .enable_ntpshm  = false,
+    .shmTime	    = {0},
+    .shmTimeInuse   = {0},
 # ifdef PPS_ENABLE
-    .shmTimePPS         = false,
+    .shmTimePPS	    = false,
 # endif /* PPS_ENABLE */
 #endif /* NTPSHM_ENABLE */
 };
 /*@ +initallelements +nullassign +nullderef @*/
 
+static volatile sig_atomic_t signalled;
+
 static void onsig(int sig)
 {
-    /* go back to the default signal action until we can reset properly */
-    (void)signal(sig, SIG_DFL);
-    longjmp(restartbuf, sig+1);
+    /* just set a variable, and deal with it in the main loop */
+    signalled = (sig_atomic_t)sig;
 }
 
 static int daemonize(void)
@@ -181,29 +196,36 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 
 static void usage(void)
 {
-    (void)printf("usage: gpsd [-n] [-N] [-D n] [-F sockfile] [-P pidfile] [-S port] [-h] device...\n\
+    struct gps_type_t **dp;
+
+    (void)printf("usage: gpsd [-b] [-n] [-N] [-D n] [-F sockfile] [-P pidfile] [-S port] [-h] device...\n\
   Options include: \n\
-  -n                            = don't wait for client connects to poll GPS\n\
-  -N                            = don't go into background\n\
-  -F sockfile                   = specify control socket location\n\
-  -P pidfile              	= set file to record process ID \n\
-  -D integer (default 0)  	= set debug level \n\
-  -S integer (default %s)	= set port for daemon \n\
-  -h                     	= help message \n\
-  -V                            = emit version and exit.\n\
+  -b		     	    = bluetooth-safe: open data sources read-only\n\
+  -n			    = don't wait for client connects to poll GPS\n\
+  -N			    = don't go into background\n\
+  -F sockfile		    = specify control socket location\n\
+  -P pidfile	      	    = set file to record process ID \n\
+  -D integer (default 0)    = set debug level \n\
+  -S integer (default %s) = set port for daemon \n\
+  -h		     	    = help message \n\
+  -V			    = emit version and exit.\n\
 A device may be a local serial device for GPS input, or a URL of the form:\n\
      [{dgpsip|ntrip}://][user:passwd@]host[:port][/stream]\n\
-in which case it specifies an input source for DGPS or ntrip data.\n",
-
+in which case it specifies an input source for DGPS or ntrip data.\n\
+\n\
+The following driver types are compiled into this gpsd instance:\n",
 	   DEFAULT_GPSD_PORT);
+    for (dp = gpsd_drivers; *dp; dp++) {
+	(void)printf("    %s\n", (*dp)->typename);
+    }
 }
 
 static int passivesock(char *service, char *protocol, int qlen)
 {
     struct servent *pse;
-    struct protoent *ppe ;
+    struct protoent *ppe ;	/* splint has a bug here */
     struct sockaddr_in sin;
-    int s, type, one = 1;
+    int s, type, proto, one = 1;
 
     /*@ -mustfreefresh @*/
     memset((char *) &sin, 0, sizeof(sin));
@@ -216,15 +238,15 @@ static int passivesock(char *service, char *protocol, int qlen)
 	gpsd_report(LOG_ERROR, "Can't get \"%s\" service entry.\n", service);
 	return -1;
     }
-    if ((ppe = getprotobyname(protocol)) == NULL) {
-	gpsd_report(LOG_ERROR, "Can't get \"%s\" protocol entry.\n", protocol);
-	return -1;
-    }
-    if (strcmp(protocol, "udp") == 0)
+    ppe = getprotobyname(protocol);
+    if (strcmp(protocol, "udp") == 0) {
 	type = SOCK_DGRAM;
-    else
+	/*@i@*/proto = (ppe) ? ppe->p_proto : IPPROTO_UDP;
+    } else {
 	type = SOCK_STREAM;
-    if ((s = socket(PF_INET, type, /*@i1@*/ppe->p_proto)) < 0) {
+	/*@i@*/proto = (ppe) ? ppe->p_proto : IPPROTO_TCP;
+    }
+    if ((s = socket(PF_INET, type, /*@i1@*/proto)) < 0) {
 	gpsd_report(LOG_ERROR, "Can't create socket\n");
 	return -1;
     }
@@ -234,9 +256,9 @@ static int passivesock(char *service, char *protocol, int qlen)
     }
     if (bind(s, (struct sockaddr *) &sin, (int)sizeof(sin)) < 0) {
 	gpsd_report(LOG_ERROR, "Can't bind to port %s\n", service);
-        if (errno == EADDRINUSE) {
-                gpsd_report(LOG_ERROR, "Maybe gpsd is already running!\n");
-        }
+	if (errno == EADDRINUSE) {
+		gpsd_report(LOG_ERROR, "Maybe gpsd is already running!\n");
+	}
 	return -1;
     }
     if (type == SOCK_STREAM && listen(s, qlen) < 0) {
@@ -282,20 +304,15 @@ static int filesock(char *filename)
 #define MAXDEVICES	4
 #endif
 
-/* 
- * Note: stdin/stdout, logging, and the control socket may eat several
- * file descriptors, so don't set this too low. 16 should probably be
- * the minimum.
- */
 #ifdef LIMITED_MAX_CLIENTS
 #define MAXSUBSCRIBERS LIMITED_MAX_CLIENTS
 #else
-/* subscriber structure is small enough that there's no need to limit this */ 
+/* subscriber structure is small enough that there's no need to limit this */
 #define MAXSUBSCRIBERS	FD_SETSIZE
 #endif
 
 /*
- * Multi-session support requires us to have two arrays, one of GPS 
+ * Multi-session support requires us to have two arrays, one of GPS
  * devices currently available and one of client sessions.  The number
  * of slots in each array is limited by the maximum number of client
  * sessions we can have open.
@@ -307,8 +324,8 @@ static struct gps_device_t channels[MAXDEVICES];
 #define syncing(chp)	(chp->gpsdata.gps_fd>-1&& chp->packet_type==BAD_PACKET)
 
 static struct subscriber_t {
-    int fd;				/* client file descriptor. -1 if unused */
-    double active;			/* when subscriber last polled for data */
+    int fd;			/* client file descriptor. -1 if unused */
+    double active;		/* when subscriber last polled for data */
     bool tied;				/* client set device with F */
     bool watcher;			/* is client in watcher mode? */
     int raw;				/* is client in raw mode? */
@@ -325,11 +342,11 @@ static void adjust_max_fd(int fd, bool on)
     if (on) {
 	if (fd > maxfd)
 	    maxfd = fd;
-    } 
+    }
 #if !defined(LIMITED_MAX_DEVICES) && !defined(LIMITED_MAX_CLIENT_FD)
     /*
      * I suspect there could be some weird interactions here if
-     * either of these were set lower than FD_SETSIZE.  We'll avoid 
+     * either of these were set lower than FD_SETSIZE.  We'll avoid
      * potential bugs by not scavenging in this case at all -- should
      * be OK, as the use case for limiting is SBCs where the limits
      * will be very low (typically 1) and the maximum size of fd
@@ -373,16 +390,21 @@ static /*@null@*/ /*@observer@*/ struct subscriber_t* allocate_client(void)
 {
     int cfd;
     for (cfd = 0; cfd < MAXSUBSCRIBERS; cfd++) {
-	if (subscribers[cfd].device == NULL) 
+	if (subscribers[cfd].fd <= 0 ) {
+	    subscribers[cfd].fd = cfd; /* mark subscriber as allocated */
 	    return &subscribers[cfd];
+	}
     }
     return NULL;
 }
 
 static void detach_client(struct subscriber_t *sub)
 {
+    char *c_ip = sock2ip(sub->fd);
+    (void)shutdown(sub->fd, SHUT_RDWR);
     (void)close(sub->fd);
-    gpsd_report(LOG_PROG, "detaching %d in detach_client\n", sub_index(sub));
+    gpsd_report(LOG_INF, "detaching %s (sub%d, fd %d) in detach_client\n",
+	c_ip, sub_index(sub), sub->fd);
     FD_CLR(sub->fd, &all_fds);
     adjust_max_fd(sub->fd, false);
     sub->raw = 0;
@@ -405,16 +427,24 @@ static ssize_t throttled_write(struct subscriber_t *sub, char *buf, ssize_t len)
 	    char *cp, buf2[MAX_PACKET_LENGTH*3];
 	    buf2[0] = '\0';
 	    for (cp = buf; cp < buf + len; cp++)
-		(void)snprintf(buf2 + strlen(buf2), 
+		(void)snprintf(buf2 + strlen(buf2),
 			       sizeof(buf2)-strlen(buf2),
 			      "%02x", (unsigned int)(*cp & 0xff));
 	    gpsd_report(LOG_IO, "=> client(%d): =%s\r\n", sub_index(sub), buf2);
 	}
     }
 
-    if ((status = write(sub->fd, buf, (size_t)len)) > -1)
+    status = write(sub->fd, buf, (size_t)len);
+    if (status == len )
 	return status;
-    if (errno == EBADF)
+    else if (status > -1) {
+	gpsd_report(LOG_INF, "short write disconnecting client(%d)\n",
+	    sub_index(sub));
+	detach_client(sub);
+	return 0;
+    } else if (errno == EAGAIN || errno == EINTR)
+	return 0; /* no data written, and errno says to retry */
+     else if (errno == EBADF)
 	gpsd_report(LOG_WARN, "client(%d) has vanished.\n", sub_index(sub));
     else if (errno == EWOULDBLOCK && timestamp() - sub->active > NOREAD_TIMEOUT)
 	gpsd_report(LOG_INF, "client(%d) timed out.\n", sub_index(sub));
@@ -440,7 +470,7 @@ static void notify_watchers(struct gps_device_t *device, char *sentence, ...)
 	    (void)throttled_write(sub, buf, (ssize_t)strlen(buf));
 }
 
-static void raw_hook(struct gps_data_t *ud, 
+static void raw_hook(struct gps_data_t *ud,
 		     char *sentence, size_t len, int level)
 /* hook to be executed on each incoming packet */
 {
@@ -448,7 +478,7 @@ static void raw_hook(struct gps_data_t *ud,
 
     for (sub = subscribers; sub < subscribers + MAXSUBSCRIBERS; sub++) {
 	/* copy raw NMEA sentences from GPS to clients in raw mode */
-	if (sub->raw == level && 
+	if (sub->raw == level &&
 	    sub->device!=NULL &&
 	    strcmp(ud->gps_device, sub->device->gpsdata.gps_device)==0)
 	    (void)throttled_write(sub, sentence, (ssize_t)len);
@@ -488,7 +518,7 @@ static /*@null@*/ struct gps_device_t *open_device(char *device_name)
     for (chp = channels; chp < channels + MAXDEVICES; chp++)
 	if (!allocated_channel(chp)){
 	    goto found;
-        }
+	}
     return NULL;
 found:
     gpsd_init(chp, &context, device_name);
@@ -527,7 +557,7 @@ static bool allocation_policy(struct gps_device_t *channel,
 	return true;
     else if (user->requires==RTCM104 && (channel->packet.type==RTCM_PACKET))
 	return true;
-    else if (user->requires == GPS 
+    else if (user->requires == GPS
 	     && (channel->packet.type!=RTCM_PACKET) && (channel->packet.type!=BAD_PACKET))
 	return true;
     else
@@ -562,12 +592,12 @@ static bool assign_channel(struct subscriber_t *user)
     }
 
     /* and open that device */
-    if (user->device->gpsdata.gps_fd != -1) 
+    if (user->device->gpsdata.gps_fd != -1)
 	gpsd_report(LOG_PROG,"client(%d): channel %d already active.\n",
 		    user-subscribers, user->device->gpsdata.gps_fd);
     else {
 	if (gpsd_activate(user->device, true) < 0) {
-	    
+
 	    gpsd_report(LOG_ERROR, "client(%d): channel activation failed.\n", user-subscribers);
 	    return false;
 	} else {
@@ -576,7 +606,7 @@ static bool assign_channel(struct subscriber_t *user)
 	    adjust_max_fd(user->device->gpsdata.gps_fd, true);
 	    if (user->watcher && !user->tied) {
 		(void)write(user->fd, "GPSD,F=", 7);
-		(void)write(user->fd, 
+		(void)write(user->fd,
 			    user->device->gpsdata.gps_device,
 			    strlen(user->device->gpsdata.gps_device));
 		(void)write(user->fd, "\r\n", 2);
@@ -586,7 +616,7 @@ static bool assign_channel(struct subscriber_t *user)
 
     if (user->watcher && was_unassigned) {
 	char buf[NMEA_MAX];
-	(void)snprintf(buf, sizeof(buf), "GPSD,X=%f,I=%s\r\n", 
+	(void)snprintf(buf, sizeof(buf), "GPSD,X=%f,I=%s\r\n",
 		       timestamp(), gpsd_id(user->device));
 	(void)write(user->fd, buf, strlen(buf));
     }
@@ -645,29 +675,29 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 	switch (toupper(*p++)) {
 	case 'A':
 	    if (assign_channel(sub) && have_fix(sub) && sub->fixbuffer.mode == MODE_3D)
-		(void)snprintf(phrase, sizeof(phrase), ",A=%.3f", 
+		(void)snprintf(phrase, sizeof(phrase), ",A=%.3f",
 			sub->fixbuffer.altitude);
 	    else
 		(void)strlcpy(phrase, ",A=?", BUFSIZ);
 	    break;
 	case 'B':		/* change baud rate (SiRF/Zodiac only) */
 #ifndef FIXED_PORT_SPEED
-	    if (assign_channel(sub) && sub->device->device_type!=NULL && *p=='=' && privileged_user(sub)) {
+	    if (assign_channel(sub) && sub->device->device_type!=NULL && *p=='=' && privileged_user(sub) && !context.readonly) {
 		i = atoi(++p);
 		while (isdigit(*p)) p++;
 #ifdef ALLOW_RECONFIGURE
 		if (sub->device->device_type->speed_switcher)
 		    if (sub->device->device_type->speed_switcher(sub->device, (unsigned)i)) {
-			/* 
+			/*
 			 * Allow the control string time to register at the
-			 * GPS before we do the baud rate switch, which 
+			 * GPS before we do the baud rate switch, which
 			 * effectively trashes the UART's buffer.
 			 *
 			 * This definitely fails below 40 milliseconds on a
-			 * BU-303b. 50ms is also verified by Chris Kuethe on 
-			 *        Pharos iGPS360 + GSW 2.3.1ES + prolific
-			 *        Rayming TN-200 + GSW 2.3.1 + ftdi
-			 *        Rayming TN-200 + GSW 2.3.2 + ftdi
+			 * BU-303b. 50ms is also verified by Chris Kuethe on
+			 *	Pharos iGPS360 + GSW 2.3.1ES + prolific
+			 *	Rayming TN-200 + GSW 2.3.1 + ftdi
+			 *	Rayming TN-200 + GSW 2.3.2 + ftdi
 			 * so it looks pretty solid.
 			 *
 			 * The minimum delay time is probably constant
@@ -687,9 +717,9 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 			/* zero parity breaks the next snprintf */
 			sub->device->gpsdata.parity = (unsigned)'N';
 		}
-		(void)snprintf(phrase, sizeof(phrase), ",B=%d %d %c %u", 
+		(void)snprintf(phrase, sizeof(phrase), ",B=%d %d %c %u",
 		    (int)gpsd_get_speed(&sub->device->ttyset),
-			9 - sub->device->gpsdata.stopbits, 
+			9 - sub->device->gpsdata.stopbits,
 			(int)sub->device->gpsdata.parity,
 			sub->device->gpsdata.stopbits);
 	    } else {
@@ -710,18 +740,18 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 				dev->cycle = cycle;
 		}
 		if (dev->rate_switcher == NULL)
-		    (void)snprintf(phrase, sizeof(phrase), 
+		    (void)snprintf(phrase, sizeof(phrase),
 				   ",C=%.2f", dev->cycle);
 		else
-		    (void)snprintf(phrase, sizeof(phrase), 
+		    (void)snprintf(phrase, sizeof(phrase),
 				   ",C=%.2f %.2f", dev->cycle, mincycle);
 	    }
 	    break;
 	case 'D':
 	    (void)strlcpy(phrase, ",D=", BUFSIZ);
 	    if (assign_channel(sub) && isnan(sub->fixbuffer.time)==0)
-		(void)unix_to_iso8601(sub->fixbuffer.time, 
-				phrase+3, (int)(sizeof(phrase)-3));
+		(void)unix_to_iso8601(sub->fixbuffer.time,
+				phrase+3, sizeof(phrase)-3);
 	    else
 		(void)strlcat(phrase, "?", BUFSIZ);
 	    break;
@@ -761,7 +791,7 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 	    }
 	    /*@ +branchstate @*/
 	    if (sub->device != NULL)
-		(void)snprintf(phrase, sizeof(phrase), ",F=%s", 
+		(void)snprintf(phrase, sizeof(phrase), ",F=%s",
 			 sub->device->gpsdata.gps_device);
 	    else
 		(void)strlcpy(phrase, ",F=?", BUFSIZ);
@@ -787,7 +817,7 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 	    break;
 	case 'I':
 	    if (assign_channel(sub) && sub->device->device_type!=NULL) {
-		(void)snprintf(phrase, sizeof(phrase), ",I=%s", 
+		(void)snprintf(phrase, sizeof(phrase), ",I=%s",
 			       gpsd_id(sub->device));
 	    } else
 		(void)strlcpy(phrase, ",I=?", BUFSIZ);
@@ -831,7 +861,7 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 	    else if (!sub->device->device_type->mode_switcher)
 		(void)strlcpy(phrase, ",N=0", BUFSIZ);
 #ifdef ALLOW_RECONFIGURE
-	    else if (privileged_user(sub)) {
+	    else if (privileged_user(sub) && !context.readonly) {
 		if (*p == '=') ++p;
 		if (*p == '1' || *p == '+') {
 		    sub->device->device_type->mode_switcher(sub->device, 1);
@@ -889,34 +919,34 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 		else
 		    (void)strlcat(phrase, " ?", BUFSIZ);
 		if (isnan(sub->fixbuffer.eph)==0)
-		    (void)snprintf(phrase+strlen(phrase), 
+		    (void)snprintf(phrase+strlen(phrase),
 				   sizeof(phrase)-strlen(phrase),
 				  " %.2f",  sub->fixbuffer.eph);
 		else
 		    (void)strlcat(phrase, " ?", BUFSIZ);
 		if (isnan(sub->fixbuffer.epv)==0)
-		    (void)snprintf(phrase+strlen(phrase), 
+		    (void)snprintf(phrase+strlen(phrase),
 				   sizeof(phrase)-strlen(phrase),
 				   " %.2f",  sub->fixbuffer.epv);
 		else
 		    (void)strlcat(phrase, " ?", BUFSIZ);
 		if (isnan(sub->fixbuffer.track)==0)
-		    (void)snprintf(phrase+strlen(phrase), 
+		    (void)snprintf(phrase+strlen(phrase),
 				   sizeof(phrase)-strlen(phrase),
 				   " %.4f %.3f",
-				   sub->fixbuffer.track, 
+				   sub->fixbuffer.track,
 				   sub->fixbuffer.speed);
 		else
 		    (void)strlcat(phrase, " ? ?", BUFSIZ);
 		if (isnan(sub->fixbuffer.climb)==0)
 		    (void)snprintf(phrase+strlen(phrase),
 				   sizeof(phrase)-strlen(phrase),
-				   " %.3f", 
+				   " %.3f",
 				   sub->fixbuffer.climb);
 		else
 		    (void)strlcat(phrase, " ?", BUFSIZ);
 		if (isnan(sub->fixbuffer.epd)==0)
-		    (void)snprintf(phrase+strlen(phrase), 
+		    (void)snprintf(phrase+strlen(phrase),
 				   sizeof(phrase)-strlen(phrase),
 				   " %.4f",
 				   sub->fixbuffer.epd);
@@ -925,41 +955,41 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 		if (isnan(sub->fixbuffer.eps)==0)
 		    (void)snprintf(phrase+strlen(phrase),
 			     sizeof(phrase)-strlen(phrase),
-			     " %.2f", sub->fixbuffer.eps);		    
+			     " %.2f", sub->fixbuffer.eps);
 		else
 		    (void)strlcat(phrase, " ?", BUFSIZ);
 		if (isnan(sub->fixbuffer.epc)==0)
 		    (void)snprintf(phrase+strlen(phrase),
 			     sizeof(phrase)-strlen(phrase),
-			     " %.2f", sub->fixbuffer.epc);		    
+			     " %.2f", sub->fixbuffer.epc);
 		else
 		    (void)strlcat(phrase, " ?", BUFSIZ);
 		if (sub->fixbuffer.mode > 0)
 		    (void)snprintf(phrase+strlen(phrase),
 			     sizeof(phrase)-strlen(phrase),
-			     " %d", sub->fixbuffer.mode);		    
+			     " %d", sub->fixbuffer.mode);
 		else
 		    (void)strlcat(phrase, " ?", BUFSIZ);
 	    }
 	    break;
 	case 'P':
 	    if (assign_channel(sub) && have_fix(sub))
-		(void)snprintf(phrase, sizeof(phrase), ",P=%.6f %.6f", 
-			sub->fixbuffer.latitude, 
+		(void)snprintf(phrase, sizeof(phrase), ",P=%.6f %.6f",
+			sub->fixbuffer.latitude,
 			sub->fixbuffer.longitude);
 	    else
 		(void)strlcpy(phrase, ",P=?", BUFSIZ);
 	    break;
 	case 'Q':
-#define ZEROIZE(x)	(isnan(x)!=0 ? 0.0 : x)  
-	    if (assign_channel(sub) && 
+#define ZEROIZE(x)	(isnan(x)!=0 ? 0.0 : x)
+	    if (assign_channel(sub) &&
 		(isnan(sub->device->gpsdata.pdop)==0
 		 || isnan(sub->device->gpsdata.hdop)==0
 		 || isnan(sub->device->gpsdata.vdop)==0))
 		(void)snprintf(phrase, sizeof(phrase), ",Q=%d %.2f %.2f %.2f %.2f %.2f",
-			sub->device->gpsdata.satellites_used, 
-			ZEROIZE(sub->device->gpsdata.pdop), 
-			ZEROIZE(sub->device->gpsdata.hdop), 
+			sub->device->gpsdata.satellites_used,
+			ZEROIZE(sub->device->gpsdata.pdop),
+			ZEROIZE(sub->device->gpsdata.hdop),
 			ZEROIZE(sub->device->gpsdata.vdop),
 			ZEROIZE(sub->device->gpsdata.tdop),
 			ZEROIZE(sub->device->gpsdata.gdop));
@@ -1042,7 +1072,7 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 		(void)snprintf(phrase, sizeof(phrase), ",W=1");
 	    }
 	    break;
-        case 'X':
+	case 'X':
 	    if (assign_channel(sub) && sub->device != NULL)
 		(void)snprintf(phrase, sizeof(phrase), ",X=%f", sub->device->gpsdata.online);
 	    else
@@ -1057,7 +1087,7 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 		else
 		    (void)strlcat(phrase, "-", BUFSIZ);
 		if (isnan(sub->device->gpsdata.sentence_time)==0)
-		    (void)snprintf(phrase+strlen(phrase), 
+		    (void)snprintf(phrase+strlen(phrase),
 				   sizeof(phrase)-strlen(phrase),
 				   " %.3f ",
 				   sub->device->gpsdata.sentence_time);
@@ -1067,7 +1097,7 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 		for (i = 0; i < sub->device->gpsdata.satellites; i++)
 		    if (sub->device->gpsdata.PRN[i])
 			reported++;
-		(void)snprintf(phrase+strlen(phrase), 
+		(void)snprintf(phrase+strlen(phrase),
 			       sizeof(phrase)-strlen(phrase),
 			       "%d:", reported);
 		for (i = 0; i < sub->device->gpsdata.satellites; i++) {
@@ -1078,10 +1108,10 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 			    break;
 			}
 		    if (sub->device->gpsdata.PRN[i]) {
-			(void)snprintf(phrase+strlen(phrase), 
+			(void)snprintf(phrase+strlen(phrase),
 				      sizeof(phrase)-strlen(phrase),
-				      "%d %d %d %d %d:", 
-				      sub->device->gpsdata.PRN[i], 
+				      "%d %d %d %d %d:",
+				      sub->device->gpsdata.PRN[i],
 				      sub->device->gpsdata.elevation[i],sub->device->gpsdata.azimuth[i],
 				      sub->device->gpsdata.ss[i],
 				      used);
@@ -1094,11 +1124,11 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 		(void)strlcpy(phrase, ",Y=?", BUFSIZ);
 	    break;
 	case 'Z':
-	    (void)assign_channel(sub); 
+	    (void)assign_channel(sub);
 	    if (*p == '=') ++p;
 	    if (sub->device == NULL) {
 		(void)snprintf(phrase, sizeof(phrase), ",Z=?");
-		p++;		
+		p++;
 	    } else if (*p == '1' || *p == '+') {
 		sub->device->gpsdata.profiling = true;
 		gpsd_report(LOG_INF, "client(%d) turned on profiling mode\n", sub_index(sub));
@@ -1116,7 +1146,7 @@ static int handle_gpsd_request(struct subscriber_t* sub, char *buf, int buflen)
 			       (int)sub->device->gpsdata.profiling);
 	    }
 	    break;
-        case '$':
+	case '$':
 	    if (!assign_channel(sub))
 		(void)strlcpy(phrase, ",$=?", BUFSIZ);
 	    else if (sub->device->gpsdata.sentence_time!=0)
@@ -1172,8 +1202,8 @@ static void handle_control(int sfd, char *buf)
 	    for (cfd = 0; cfd < MAXSUBSCRIBERS; cfd++)
 		if (subscribers[cfd].device == chp)
 		    subscribers[cfd].device = NULL;
-	    chp->gpsdata.gps_fd = -1;	/* device is already disconnected */
 	    gpsd_wrap(chp);
+	    chp->gpsdata.gps_fd = -1;	/* device is already disconnected */
 	    /*@i@*/free_channel(chp);	/* modifying observer storage */
 	    (void)write(sfd, "OK\n", 3);
 	} else
@@ -1216,16 +1246,16 @@ int main(int argc, char *argv[])
     static char *pid_file = NULL;
     static int st, csock = -1;
     static gps_mask_t changed;
-    static char *gpsd_service = NULL; 
+    static char *gpsd_service = NULL;
 #ifdef RTCM104_SERVICE
-    static char *rtcm_service = NULL; 
+    static char *rtcm_service = NULL;
     static int nsock;
 #endif /* RTCM104_SERVICE */
     static char *control_socket = NULL;
     struct gps_device_t *channel;
     struct sockaddr_in fsin;
     fd_set rfds, control_fds;
-    int i, option, msock, cfd, dfd; 
+    int i, option, msock, cfd, dfd;
     bool go_background = true;
     struct timeval tv;
 #ifdef RTCM104_SERVICE
@@ -1237,8 +1267,11 @@ int main(int argc, char *argv[])
     pthread_mutex_init(&report_mutex, NULL);
 #endif /* PPS_ENABLE */
 
+#ifdef HAVE_SETLOCALE
+    (void)setlocale(LC_NUMERIC, "C");
+#endif
     debuglevel = 0;
-    while ((option = getopt(argc, argv, "F:D:S:dfhNnpP:V"
+    while ((option = getopt(argc, argv, "F:D:S:bhNnP:V"
 #ifdef RTCM104_SERVICE
 			    "R:"
 #endif /* RTCM104_SERVICE */
@@ -1253,6 +1286,9 @@ int main(int argc, char *argv[])
 	case 'N':
 	    go_background = false;
 	    break;
+	case 'b':
+	    context.readonly = true;
+	    break;
 #ifdef RTCM104_SERVICE
 	case 'R':
 	    rtcm_service = optarg;
@@ -1264,11 +1300,6 @@ int main(int argc, char *argv[])
 	case 'n':
 	    nowait = true;
 	    break;
-	case 'd':
-	case 'f':
-	case 'p':
-	    /* skip this option, treat following as argument */ 
-	    break;
 	case 'P':
 	    pid_file = optarg;
 	    break;
@@ -1369,22 +1400,23 @@ int main(int argc, char *argv[])
 	struct stat stb;
 
 	/* make default devices accessible even after we drop privileges */
-	for (i = optind; i < argc; i++) 
+	for (i = optind; i < argc; i++)
 	    if (stat(argv[i], &stb) == 0)
 		(void)chmod(argv[i], stb.st_mode|S_IRGRP|S_IWGRP);
 	/*
 	 * Drop privileges.  Up to now we've been running as root.  Instead,
-	 * set the user ID to 'nobody' and the group ID to the owning group 
-	 * of a prototypical TTY device.  This limits the scope of any
-	 * compromises in the code.  It requires that all GPS devices have
-	 * their group read/write permissions set.
+	 * set the user ID to 'nobody' (or whatever the --enable-gpsd-user
+	 * is) and the group ID to the owning group of a prototypical TTY
+	 * device. This limits the scope of any compromises in the code.
+	 * It requires that all GPS devices have their group read/write
+	 * permissions set.
 	 */
 	if ((optind<argc&&stat(argv[optind], &stb)==0)||stat(PROTO_TTY,&stb)==0) {
 	    gpsd_report(LOG_PROG, "changing to group %d\n", stb.st_gid);
 	    if (setgid(stb.st_gid) != 0)
 		gpsd_report(LOG_ERROR, "setgid() failed, errno %s\n", strerror(errno));
 	}
-	pw = getpwnam("nobody");
+	pw = getpwnam( GPSD_USER );
 	if (pw)
 	    (void)seteuid(pw->pw_uid);
     }
@@ -1403,19 +1435,11 @@ int main(int argc, char *argv[])
 	    if (allocated_channel(&channels[dfd]))
 		(void)gpsd_wrap(&channels[dfd]);
 	}
-	if (st == SIGHUP+1)
-	    gpsd_report(LOG_WARN, "gpsd restarted by SIGHUP\n");
-	else if (st > 0) {
-	    gpsd_report(LOG_WARN, "Received terminating signal %d. Exiting...\n",st-1);
-	    if (control_socket)
-		(void)unlink(control_socket);
-	    if (pid_file)
-		(void)unlink(pid_file);
-	    exit(10 + st);
-	}
+	gpsd_report(LOG_WARN, "gpsd restarted by SIGHUP\n");
     }
 
     /* Handle some signals */
+    signalled = 0;
     (void)signal(SIGHUP, onsig);
     (void)signal(SIGINT, onsig);
     (void)signal(SIGTERM, onsig);
@@ -1434,18 +1458,18 @@ int main(int argc, char *argv[])
     if (time(NULL) < START_SUBFRAME)
 	context.valid |= LEAP_SECOND_VALID;
 
-    for (i = optind; i < argc; i++) { 
+    for (i = optind; i < argc; i++) {
 	struct gps_device_t *device = open_device(argv[i]);
 	if (!device) {
 	    gpsd_report(LOG_ERROR, "GPS device %s nonexistent or can't be read\n", argv[i]);
 	}
     }
 
-    for (;;) {
-        (void)memcpy((char *)&rfds, (char *)&all_fds, sizeof(rfds));
+    while (0 == signalled) {
+	(void)memcpy((char *)&rfds, (char *)&all_fds, sizeof(rfds));
 
 	gpsd_report(LOG_RAW+2, "select waits\n");
-	/* 
+	/*
 	 * Poll for user commands or GPS data.  The timeout doesn't
 	 * actually matter here since select returns whenever one of
 	 * the file descriptors in the set goes ready.  The point
@@ -1469,13 +1493,13 @@ int main(int argc, char *argv[])
 	    dbuf[0] = '\0';
 	    for (sub = subscribers; sub < subscribers + MAXSUBSCRIBERS; sub++)
 		if (FD_ISSET(sub->fd, &all_fds))
-		    (void)snprintf(dbuf + strlen(dbuf), 
+		    (void)snprintf(dbuf + strlen(dbuf),
 				   sizeof(dbuf)-strlen(dbuf),
 				   " %d", sub->fd);
 	    strlcat(dbuf, "} -> {", BUFSIZ);
 	    for (sub = subscribers; sub < subscribers + MAXSUBSCRIBERS; sub++)
 		if (FD_ISSET(sub->fd, &rfds))
-		    (void)snprintf(dbuf + strlen(dbuf), 
+		    (void)snprintf(dbuf + strlen(dbuf),
 				   sizeof(dbuf)-strlen(dbuf),
 				   " %d", sub->fd);
 	    gpsd_report(LOG_RAW, "Polling descriptor set: {%s}\n", dbuf);
@@ -1485,6 +1509,7 @@ int main(int argc, char *argv[])
 	/* always be open to new client connections */
 	if (FD_ISSET(msock, &rfds)) {
 	    socklen_t alen = (socklen_t)sizeof(fsin);
+	    char *c_ip;
 	    /*@i1@*/int ssock = accept(msock, (struct sockaddr *) &fsin, &alen);
 
 	    if (ssock < 0)
@@ -1495,19 +1520,22 @@ int main(int argc, char *argv[])
 
 		if (opts >= 0)
 		    (void)fcntl(ssock, F_SETFL, opts | O_NONBLOCK);
-		gpsd_report(LOG_INF, "client connect on %d\n", ssock);
 
+		c_ip = sock2ip(ssock);
 		client = allocate_client();
 		if (client == NULL) {
-		    gpsd_report(LOG_ERROR, "No client subscriber slots available!\n");
+		    gpsd_report(LOG_ERROR, "Client %s connect on fd %d -"
+			"no subscriber slots available\n", c_ip, ssock);
 		    (void)close(ssock);
 		} else {
-			FD_SET(ssock, &all_fds);
-			adjust_max_fd(ssock, true);
-			client->fd = ssock;
-			client->active = timestamp();
-			client->tied = false;
-			client->requires = ANY;
+		    FD_SET(ssock, &all_fds);
+		    adjust_max_fd(ssock, true);
+		    client->fd = ssock;
+		    client->active = timestamp();
+		    client->tied = false;
+		    client->requires = ANY;
+		    gpsd_report(LOG_INF, "client %s (%d) connect on fd %d\n",
+			c_ip, sub_index(client), ssock);
 		}
 	    }
 	    FD_CLR(msock, &rfds);
@@ -1516,6 +1544,7 @@ int main(int argc, char *argv[])
 #ifdef RTCM104_SERVICE
 	/* also to RTCM client connections */
 	if (FD_ISSET(nsock, &rfds)) {
+	    char *c_ip;
 	    socklen_t alen = (socklen_t)sizeof(fsin);
 	    /*@i1@*/int ssock = accept(nsock, (struct sockaddr *)&fsin, &alen);
 
@@ -1527,10 +1556,11 @@ int main(int argc, char *argv[])
 
 		if (opts >= 0)
 		    (void)fcntl(ssock, F_SETFL, opts | O_NONBLOCK);
-		gpsd_report(LOG_INF, "client connect on %d\n", ssock);
+		c_ip = sock2ip(ssock);
 		client = allocate_client();
 		if (client == NULL) {
-		    gpsd_report(LOG_ERROR, "No client subscriber slots available!\n");
+		    gpsd_report(LOG_ERROR, "Client %s connect on fd %d -"
+			"no subscriber slots available\n", c_ip, ssock);
 		    close(ssock);
 		} else {
 		    FD_SET(ssock, &all_fds);
@@ -1539,6 +1569,8 @@ int main(int argc, char *argv[])
 		    client->tied = false;
 		    client->requires = RTCM104;
 		    client->fd = ssock;
+		    gpsd_report(LOG_INF, "client %s (%d) connect on fd %d\n",
+			c_ip, sub_index(client), ssock);
 		}
 	    }
 	    FD_CLR(nsock, &rfds);
@@ -1605,10 +1637,8 @@ int main(int argc, char *argv[])
 		    adjust_max_fd(channel->gpsdata.gps_fd, false);
 		    gpsd_deactivate(channel);
 		} else if ((changed & ONLINE_SET) == 0) {
-/*
-		    gpsd_report(LOG_INF, "GPS is offline (%lf sec since data)\n", 
+		    gpsd_report(LOG_INF, "GPS is offline (%lf sec since data)\n",
 				timestamp() - channel->gpsdata.online);
-*/
 		    FD_CLR(channel->gpsdata.gps_fd, &all_fds);
 		    adjust_max_fd(channel->gpsdata.gps_fd, false);
 		    gpsd_deactivate(channel);
@@ -1619,7 +1649,7 @@ int main(int argc, char *argv[])
 		    if ((changed & DEVICEID_SET) != 0) {
 			char id[NMEA_MAX];
 			assert(channel->device_type != NULL);
-			(void)snprintf(id, sizeof(id), "GPSD,I=%s", 
+			(void)snprintf(id, sizeof(id), "GPSD,I=%s",
 				       channel->device_type->typename);
 			if (channel->subtype[0] != '\0') {
 			    (void)strlcat(id, " ", sizeof(id));
@@ -1638,10 +1668,12 @@ int main(int argc, char *argv[])
 			    /* don't downgrade mode if holding previous fix */
 			    if (sub->fixbuffer.mode > sub->device->gpsdata.fix.mode)
 				changed &=~ MODE_SET;
-			    gps_merge_fix(&sub->fixbuffer, 
+			    gpsd_report(LOG_PROG,
+					"transfer mask on %s: %02x\n", sub->device->gpsdata.tag, changed);
+			    gps_merge_fix(&sub->fixbuffer,
 					  changed,
 					  &sub->device->gpsdata.fix);
-			    gpsd_error_model(sub->device, 
+			    gpsd_error_model(sub->device,
 					     &sub->fixbuffer, &sub->oldfix);
 			}
 		    }
@@ -1659,7 +1691,7 @@ int main(int argc, char *argv[])
 	    for (sub = subscribers; sub < subscribers + MAXSUBSCRIBERS; sub++) {
 		/* some listeners may be in watcher mode */
 		if (sub->watcher) {
-		    char cmds[4] = ""; 
+		    char cmds[4] = "";
 		    channel->poll_times[sub - subscribers] = timestamp();
 		    if (changed &~ ONLINE_SET) {
 			if (changed & (LATLON_SET | MODE_SET))
@@ -1675,7 +1707,7 @@ int main(int argc, char *argv[])
 	    }
 #ifdef DBUS_ENABLE
 	    if (changed &~ ONLINE_SET) {
-		    if (changed & (LATLON_SET | MODE_SET)) 
+		    if (changed & (LATLON_SET | MODE_SET))
 			    send_dbus_fix (channel);
 	    }
 #endif
@@ -1696,7 +1728,7 @@ int main(int argc, char *argv[])
 
 	/* accept and execute commands for all clients */
 	for (sub = subscribers; sub < subscribers + MAXSUBSCRIBERS; sub++) {
-	    if (sub->active == 0) 
+	    if (sub->active == 0)
 		continue;
 
 	    if (FD_ISSET(sub->fd, &rfds)) {
@@ -1721,14 +1753,14 @@ int main(int argc, char *argv[])
 #endif /* RTCM104_SERVICE */
 		    {
 			if (sub->device){
-		            /*
+			    /*
 			     * when a command comes in, to update .active to
 			     * timestamp() so we don't close the connection
 			     * after POLLER_TIMEOUT seconds. This makes
 			     * POLLER_TIMEOUT useful.
 			     */
 			    sub->active = sub->device->poll_times[sub_index(sub)] = timestamp();
-                        }
+			}
 			if (handle_gpsd_request(sub, buf, buflen) < 0)
 			    detach_client(sub);
 		    }
@@ -1743,11 +1775,11 @@ int main(int argc, char *argv[])
 	}
 
 	/*
-	 * Close devices with an identified packet type but no remaining 
-	 * subscribers.  The reason the test has this particular form is 
-	 * so that, immediately after device open, we'll keep reading 
+	 * Close devices with an identified packet type but no remaining
+	 * subscribers.  The reason the test has this particular form is
+	 * so that, immediately after device open, we'll keep reading
 	 * packets until a type is identified even though there are no
-	 * subscribers yet.  We need this to happen so that subscribers 
+	 * subscribers yet.  We need this to happen so that subscribers
 	 * can later choose a device by packet type.
 	 */
 	if (!nowait)
@@ -1770,6 +1802,17 @@ int main(int argc, char *argv[])
 		}
 	    }
     }
+    /* if we make it here, we got a signal... deal with it */
+    /* restart on SIGHUP, clean up and exit otherwise */
+    if (SIGHUP == (int)signalled)
+	longjmp(restartbuf, 1);
+
+    gpsd_report(LOG_WARN, "Received terminating signal %d. Exiting...\n",signalled);
+    /* try to undo all device configurations */
+    for (dfd = 0; dfd < MAXDEVICES; dfd++) {
+	if (allocated_channel(&channels[dfd]))
+	    (void)gpsd_wrap(&channels[dfd]);
+    }
 
     if (control_socket)
 	(void)unlink(control_socket);
