@@ -1,4 +1,5 @@
-/* $Id: gpsd.h 4064 2006-12-04 05:21:11Z esr $ */
+/* This file is generated.  Do not hand-hack it! */
+
 #ifndef _gpsd_h_
 #define _gpsd_h_
 
@@ -14,11 +15,39 @@
 #endif
 #include "gps.h"
 
+/* Feature configuration switches begin here */
+#define PACKAGE_VERSION ""
+#define VERSION "2.36"
+#define ASHTECH_ENABLE 1
+/* #undef DBUS_ENABLE */
+#define EARTHMATE_ENABLE 1
+#define EVERMORE_ENABLE 1
+#define FV18_ENABLE 1
+#define GARMINTXT_ENABLE 1
+#define GARMIN_ENABLE 1
+#define GPSCLOCK_ENABLE 1
+#define ITRAX_ENABLE 1
+#define NAVCOM_ENABLE 1
+#define NMEA_ENABLE 1
+#define NTPSHM_ENABLE 1
+#define NTRIP_ENABLE 1
+#define PPS_ENABLE 1
+#define RTCM104_ENABLE 1
+#define SIRF_ENABLE 1
+/* #undef SQUELCH_ENABLE */
+/* #undef TNT_ENABLE */
+#define TRIPMATE_ENABLE 1
+#define TSIP_ENABLE 1
+#define UBX_ENABLE 1
+/* Feature configuration switches end here */
+
+/* $Id: gpsd.h-tail 4573 2007-12-18 02:23:16Z esr $ */
+
 /* Some internal capabilities depend on which drivers we're compiling. */
 #ifdef EARTHMATE_ENABLE
 #define ZODIAC_ENABLE	
 #endif
-#if defined(ZODIAC_ENABLE) || defined(SIRF_ENABLE) || defined(GARMIN_ENABLE) || defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) || defined(ITALK_ENABLE)
+#if defined(ZODIAC_ENABLE) || defined(SIRF_ENABLE) || defined(GARMIN_ENABLE) || defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) || defined(ITRAX_ENABLE) || defined(UBX_ENABLE)
 #define BINARY_ENABLE	
 #endif
 #if defined(TRIPMATE_ENABLE) || defined(BINARY_ENABLE)
@@ -27,7 +56,8 @@
 
 /* First, declarations for the packet layer... */
 
-#define NMEA_MAX	82		/* max length of NMEA sentence */
+/* XXX Some receivers (TN-200, GSW 2.3.2) emit oversized sentences (>82) */
+#define NMEA_MAX	86		/* max length of NMEA sentence */
 #define NMEA_BIG_BUF	(2*NMEA_MAX+1)	/* longer than longest NMEA sentence */
 
 /* a few bits of ISGPS magic */
@@ -41,11 +71,13 @@ enum isgpsstat_t {
 /*
  * The packet buffers need to be as long than the longest packet we
  * expect to see in any protocol, because we have to be able to hold
- * an entire packet for checksumming.  Thus, in particular, they need
- * to be as long as a SiRF MID 4 packet, 188 bytes payload plus eight bytes 
- * of header/length/checksum/trailer. 
+ * an entire packet for checksumming...
+ * First we thought it had to be big enough for a SiRF Measured Tracker
+ * Data packet (188 bytes). Then it had to be big enough for a UBX SVINFO
+ * packet (206 bytes). Now it turns out that a couple of ITALK messages are
+ * over 512 bytes. I know we like verbose output, but this is ridiculous.
  */
-#define MAX_PACKET_LENGTH	196	/* 188 + 8 */
+#define MAX_PACKET_LENGTH	516	/* 7 + 506 + 3 */
 
 /*
  * We used to define the input buffer length as MAX_PACKET_LENGTH*2+1.
@@ -55,7 +87,7 @@ enum isgpsstat_t {
  * device to be very big. I sometimes see a read of 250 characters or
  * more."
  */
-#define INPUT_BUFFER_LENGTH	1024
+#define INPUT_BUFFER_LENGTH	1536
 
 struct gps_packet_t {
     /* packet-getter internals */
@@ -70,6 +102,9 @@ struct gps_packet_t {
 #define ITALK_PACKET	6
 #define RTCM_PACKET	7
 #define GARMIN_PACKET	8
+#define NAVCOM_PACKET	9
+#define UBX_PACKET	10
+#define GARMINTXT_PACKET	11
     unsigned int state;
     size_t length;
     unsigned char inbuffer[MAX_PACKET_LENGTH*2+1];
@@ -128,16 +163,13 @@ extern void rtcm_output_magnavox(isgps30bits_t *, FILE *);
 /* this is where we choose the confidence level to use in reports */
 #define GPSD_CONFIDENCE	CEP95_SIGMA
 
-/* several places in the code try setuid. put our preferred username here */
-#define GPSD_USER "nobody"
-
 #define NTPSHMSEGS	4		/* number of NTP SHM segments */
 
 /* Some internal capabilities depend on which drivers we're compiling. */
 #ifdef EARTHMATE_ENABLE
 #define ZODIAC_ENABLE	
 #endif
-#if defined(ZODIAC_ENABLE) || defined(SIRF_ENABLE) || defined(GARMIN_ENABLE) || defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) || defined(ITALK_ENABLE)
+#if defined(ZODIAC_ENABLE) || defined(SIRF_ENABLE) || defined(GARMIN_ENABLE) || defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) || defined(ITRAX_ENABLE)
 #define BINARY_ENABLE	
 #endif
 #if defined(TRIPMATE_ENABLE) || defined(BINARY_ENABLE)
@@ -145,7 +177,8 @@ extern void rtcm_output_magnavox(isgps30bits_t *, FILE *);
 #endif
 
 struct gps_context_t {
-   int valid;				/* member validity flags */
+    int valid;				/* member validity flags */
+    bool readonly;			/* if true, never write to device */
 #define LEAP_SECOND_VALID	0x01	/* we have or don't need correction */
     /* DGPSIP status */
     bool sentdgps;			/* have we sent a DGPS report? */
@@ -228,6 +261,9 @@ struct gps_device_t {
 #endif /* NTPSHM_ENABLE */
     double mag_var;		/* Magnetic variation in degrees */  
     bool back_to_nmea;		/* back to NMEA on revert? */
+#ifdef NMEADISC
+    int ldisc;			/* current line discipline */
+#endif
     /*
      * The rest of this structure is driver-specific private storage.
      * Because the Garmin driver uses a long buffer, you can have
@@ -240,6 +276,9 @@ struct gps_device_t {
 	    int part, await;		/* for tracking GSV parts */
 	    struct tm date;
 	    double subseconds;
+#ifdef GPSCLOCK_ENABLE
+	    bool ignore_trailing_edge;
+#endif /* GPSCLOCK_ENABLE */
 	} nmea;
 #endif /* NMEA_ENABLE */
 #ifdef BINARY_ENABLE
@@ -299,6 +338,30 @@ struct gps_device_t {
 	    unsigned int Zv[ZODIAC_CHANNELS];	/* signal values (0-7) */
 	} zodiac;
 #endif /* ZODIAC_ENABLE */
+#ifdef UBX_ENABLE
+	struct {
+            unsigned int gps_week;
+    	} ubx;
+#endif /* UBX_ENABLE */
+#ifdef NAVCOM_ENABLE
+	struct {
+	    u_int8_t physical_port;
+	    bool warned;
+	} navcom;
+#endif /* NAVCOM_ENABLE */
+	/*
+	 * This is not conditionalized on RTCM104_ENABLE because we need to
+	 * be able to build rtcmdecode even when RTCM support is not
+	 * configured in the daemon.  It doesn't take up extra space.
+	 */
+	struct {
+	    /* ISGPS200 decoding */
+	    bool            locked;
+	    int             curr_offset;
+	    isgps30bits_t   curr_word;
+	    isgps30bits_t   buf[RTCM_WORDS_MAX];
+	    unsigned int    bufindex;
+	} isgps;
 #endif /* BINARY_ENABLE */
     } driver;
 };
@@ -328,7 +391,9 @@ ssize_t pass_rtcm(struct gps_device_t *, char *, size_t);
 
 extern gps_mask_t sirf_parse(struct gps_device_t *, unsigned char *, size_t);
 extern gps_mask_t evermore_parse(struct gps_device_t *, unsigned char *, size_t);
+extern gps_mask_t navcom_parse(struct gps_device_t *, unsigned char *, size_t);
 extern gps_mask_t garmin_ser_parse(struct gps_device_t *);
+extern gps_mask_t garmintxt_parse(struct gps_device_t *);
 
 extern bool dgnss_url(char *);
 extern int dgnss_open(struct gps_context_t *, char *);
@@ -349,7 +414,7 @@ extern void ntrip_report(struct gps_device_t *);
 extern void gpsd_tty_init(struct gps_device_t *);
 extern int gpsd_open(struct gps_device_t *);
 extern bool gpsd_set_raw(struct gps_device_t *);
-extern bool gpsd_write(struct gps_device_t *, void const *, size_t);
+extern ssize_t gpsd_write(struct gps_device_t *, void const *, size_t);
 extern bool gpsd_next_hunt_setting(struct gps_device_t *);
 extern int gpsd_switch_driver(struct gps_device_t *, char *);
 extern void gpsd_set_speed(struct gps_device_t *, speed_t, unsigned char, unsigned int);
@@ -362,12 +427,13 @@ extern void gpsd_interpret_subframe(struct gps_device_t *, unsigned int[]);
 extern /*@ observer @*/ char *gpsd_hexdump(const void *, size_t);
 extern int gpsd_hexpack(char *, char *, int);
 extern int hex2bin(char *);
+extern void ntpd_link_activate(struct gps_device_t *session);
 extern char /*@observer@*/ *gpsd_id(/*@in@*/struct gps_device_t *);
 extern void gpsd_position_fix_dump(struct gps_device_t *, /*@out@*/char[], size_t);
-extern void gpsd_error_model(struct gps_device_t *, 
-			     struct gps_fix_t *, struct gps_fix_t *);
+extern void gpsd_error_model(struct gps_device_t *, struct gps_fix_t *, struct gps_fix_t *);
 extern void gpsd_clear_data(struct gps_device_t *);
 extern int netlib_connectsock(const char *, const char *, const char *);
+extern char /*@observer@*/ *sock2ip(int);
 
 extern void ntpshm_init(struct gps_context_t *, bool);
 extern int ntpshm_alloc(struct gps_context_t *);
@@ -414,7 +480,7 @@ extern float roundf(float x);
 /* some OSes don't have round(). fake it if need be */
 #ifndef HAVE_ROUND
 #define	round(x) ((double)rint(x))
-#define roundf(x) ((float)rintf(x))
+#define roundf(x) ((float)rintf((double)x))
 #endif /* !HAVE_ROUND */
 
 /* OpenBSD and FreeBSD and Cygwin don't seem to have NAN, NetBSD does, others? */
@@ -431,3 +497,7 @@ void cfmakeraw(struct termios *);
 #endif /* defined(__CYGWIN__) */
 
 #endif /* _gpsd_h_ */
+
+// Local variables:
+// mode: c
+// end:
