@@ -1,4 +1,4 @@
-/* $Id: libgpsd_core.c 4112 2006-12-08 15:15:01Z esr $ */
+/* $Id: libgpsd_core.c 4600 2007-12-21 02:17:01Z esr $ */
 /* libgpsd_core.c -- direct access to GPSes on serial or USB devices. */
 #include <sys/time.h>
 #include <sys/ioctl.h>
@@ -45,7 +45,7 @@ int gpsd_switch_driver(struct gps_device_t *session, char* typename)
 	    gpsd_report(LOG_PROG, "selecting %s driver...\n", (*dp)->typename);
 	    gpsd_assert_sync(session);
 	    /*@i@*/session->device_type = *dp;
-	    if (session->device_type->probe_subtype != NULL)
+	    if (!session->context->readonly && session->device_type->probe_subtype != NULL)
 		session->device_type->probe_subtype(session, session->packet.counter = 0);
 #ifdef ALLOW_RECONFIGURE
 	    if (session->enable_reconfigure 
@@ -107,15 +107,18 @@ void gpsd_deactivate(struct gps_device_t *session)
 # endif /* PPS_ENABLE */
 #ifdef ALLOW_RECONFIGURE
     if (session->enable_reconfigure 
+	&& session->device_type != NULL
 	&& session->device_type->revert != NULL) {
 	session->device_type->revert(session);
 	session->enable_reconfigure = false;
     }
 #endif /* ALLOW_RECONFIGURE */
-    if (session->back_to_nmea && session->device_type->mode_switcher != NULL)
-	session->device_type->mode_switcher(session, 0);
-    if (session->device_type!=NULL && session->device_type->wrapup!=NULL)
-	session->device_type->wrapup(session);
+    if (session->device_type!=NULL) {
+	if (session->back_to_nmea && session->device_type->mode_switcher!=NULL)
+	    session->device_type->mode_switcher(session, 0);
+	if (session->device_type->wrapup!=NULL)
+	    session->device_type->wrapup(session);
+    }
 #endif /* NTPSHM_ENABLE */
     gpsd_report(LOG_INF, "closing GPS=%s (%d)\n", 
 		session->gpsdata.gps_device, session->gpsdata.gps_fd);
@@ -144,7 +147,7 @@ static void *gpsd_ppsmonitor(void *arg)
 	    break;
 	/*@ -ignoresigns */
 
-        state = (int)((state & pps_device) != 0);
+	state = (int)((state & pps_device) != 0);
 
 	if (state == laststate) {
 	    if (++unchanged == 10) {
@@ -164,39 +167,49 @@ static void *gpsd_ppsmonitor(void *arg)
 	    /* Garmin doc says PPS is valid after four good fixes. */
 	    /*
 	     * The PPS pulse is normally a short pulse with a frequency of
-	     * 1 Hz, and the UTC second is defined by the front edge.  But we
+	     * 1 Hz, and the UTC second is defined by the front edge. But we
 	     * don't know the polarity of the pulse (different receivers
-	     * emit different polarities).  The duration variable is used to
-	     * determine which way the pulse is going.  The code assumes
+	     * emit different polarities). The duration variable is used to
+	     * determine which way the pulse is going. The code assumes
 	     * that the UTC second is changing when the signal has not
 	     * been changing for at least 800ms, i.e. it assumes the duty
 	     * cycle is at most 20%.
-             *
-             * Some GPS instead output a square wave that is 2Hz and each
-             * edge denotes the start of a second.
+	     *
+	     * Some GPS instead output a square wave that is 0.5 Hz and each
+	     * edge denotes the start of a second.
 	     */
 #define timediff(x, y)	(int)((x.tv_sec-y.tv_sec)*1000000+x.tv_usec-y.tv_usec)
 	    cycle = timediff(tv, pulse[state]);
 	    duration = timediff(tv, pulse[state == 0]);
 #undef timediff
-	    if ( 800000 > duration) {
-		/* less then 800mS, duration too short for anything */
-                gpsd_report(LOG_RAW, 
-                     "PPS pulse rejected too short. cycle: %d, duration: %d\n",
-		     cycle, duration);
+
+	    if (800000 > duration) {
+		/* less than 800mS, duration too short for anything */
+		gpsd_report(LOG_RAW, 
+		    "PPS pulse rejected too short. cycle: %d, duration: %d\n",
+		    cycle, duration);
+#ifdef GPSCLOCK_ENABLE
+		    /*
+		     * Ugly hack to cope with things like the Furuno GPSClock,
+		     * which has the odd property that you have to
+		     * ignore the trailing edge of the PPS.
+		     */
+		    if (state==1 && session->driver.nmea.ignore_trailing_edge) 
+			(void)ntpshm_pps(session, &tv);
+#endif /* GPSCLOCK_ENABLE */
 	    } else if (cycle > 999000 && cycle < 1001000 ) {
-                /* looks like PPS pulse */
+		/* looks like PPS pulse */
 		(void)ntpshm_pps(session, &tv);
 	    } else if (cycle > 1999000 && cycle < 2001000) {
-		/* looks like 2Hz square wave */
+		/* looks like 0.5 Hz square wave */
 		(void)ntpshm_pps(session, &tv);
-            } else {
-                gpsd_report(LOG_RAW, "PPS pulse rejected.  cycle: %d, duration: %d\n",
-		     cycle, duration);
+	    } else {
+		    gpsd_report(LOG_RAW, "PPS pulse rejected.  cycle: %d, duration: %d\n",
+			cycle, duration);
 	    }
 	} else {
                 gpsd_report(LOG_RAW, "PPS pulse rejected. No fix.\n");
-        }
+	}
 	/*@ -boolint @*/
 
 	pulse[state] = tv;
@@ -210,10 +223,6 @@ static void *gpsd_ppsmonitor(void *arg)
 int gpsd_activate(struct gps_device_t *session, bool reconfigurable)
 /* acquire a connection to the GPS device */
 {
-#if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
-    pthread_t pt;
-#endif /* PPS_ENABLE */
-
     if (gpsd_open(session) < 0)
 	return -1;
     else {
@@ -226,6 +235,7 @@ int gpsd_activate(struct gps_device_t *session, bool reconfigurable)
 	    if ((*dp)->probe_detect!=NULL && (*dp)->probe_detect(session)!=0) {
 		gpsd_report(LOG_PROG, "probe found %s driver...\n", (*dp)->typename);
 		session->device_type = *dp;
+		gpsd_assert_sync(session);
 		goto foundit;
 	    }
  	}
@@ -250,19 +260,9 @@ int gpsd_activate(struct gps_device_t *session, bool reconfigurable)
 	session->gpsdata.separation = NAN;
 	session->mag_var = NAN;
 
-#ifdef NTPSHM_ENABLE
-	if (session->context->enable_ntpshm)
-	    session->shmindex = ntpshm_alloc(session->context);
-#if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
-	if (session->shmindex >= 0 && session->context->shmTimePPS) {
-	    if ((session->shmTimeP = ntpshm_alloc(session->context)) >= 0)
-		/*@i1@*/(void)pthread_create(&pt,NULL,gpsd_ppsmonitor,(void *)session);
-	}
+	/* clear driver subtype field and private data union */
 	session->subtype[0] = '\0';
 	memset(&session->driver, '\0', sizeof(session->driver));
-#endif /* defined(PPS_ENABLE) && defined(TIOCMIWAIT) */
-#endif /* NTPSHM_ENABLE */
-
 	/* if we know the device type, probe for subtype and configure it */
 	if (session->device_type != NULL) {
 	    if (session->device_type->probe_subtype !=NULL)
@@ -280,6 +280,29 @@ int gpsd_activate(struct gps_device_t *session, bool reconfigurable)
 }
 /*@ +branchstate @*/
 
+void ntpd_link_activate(struct gps_device_t *session)
+{
+#if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
+    pthread_t pt;
+#endif /* defined(PPS_ENABLE) && defined(TIOCMIWAIT) */
+
+#ifdef NTPSHM_ENABLE
+    /* If we are talking to ntpd, allocate a shared-memory segment for "NMEA" time data */
+    if (session->context->enable_ntpshm)
+        session->shmindex = ntpshm_alloc(session->context);
+
+#if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
+    /* If we also have the 1pps capability, allocate a shared-memory segment for
+     * the 1pps time data and launch a thread to capture the 1pps transitions
+     */
+    if (session->shmindex >= 0 && session->context->shmTimePPS)
+        if ((session->shmTimeP = ntpshm_alloc(session->context)) >= 0)
+            /*@i1@*/(void)pthread_create(&pt,NULL,gpsd_ppsmonitor,(void *)session);
+
+#endif /* defined(PPS_ENABLE) && defined(TIOCMIWAIT) */
+#endif /* NTPSHM_ENABLE */
+}
+
 char /*@observer@*/ *gpsd_id(/*@in@*/struct gps_device_t *session)
 /* full ID of the device for reports, including subtype */
 {
@@ -345,7 +368,7 @@ void gpsd_position_fix_dump(struct gps_device_t *session,
 	    (void)strlcat(bufp, ",", len);
 	else
 	    (void)snprintf(bufp+strlen(bufp), len-strlen(bufp), 
-			   "%.1f,M,", session->gpsdata.fix.altitude);
+			   "%.2f,M,", session->gpsdata.fix.altitude);
 	if (isnan(session->gpsdata.separation))
 	    (void)strlcat(bufp, ",", len);
 	else
@@ -370,8 +393,13 @@ static void gpsd_transit_fix_dump(struct gps_device_t *session,
     struct tm tm;
     time_t intfixtime;
 
-    intfixtime = (time_t)session->gpsdata.fix.time;
-    (void)gmtime_r(&intfixtime, &tm);
+    tm.tm_mday = tm.tm_mon = tm.tm_year = tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
+    if (isnan(session->gpsdata.fix.time)==0) {
+	intfixtime = (time_t)session->gpsdata.fix.time;
+	(void)gmtime_r(&intfixtime, &tm);
+	tm.tm_mon++;
+	tm.tm_year %= 100;
+    }
     /*@ -usedef @*/
     (void)snprintf(bufp, len,
 	    "$GPRMC,%02d%02d%02d,%c,%09.4f,%c,%010.4f,%c,%.4f,%.3f,%02d%02d%02d,,",
@@ -383,11 +411,13 @@ static void gpsd_transit_fix_dump(struct gps_device_t *session,
 	    ((session->gpsdata.fix.latitude > 0) ? 'N' : 'S'),
 	    degtodm(fabs(session->gpsdata.fix.longitude)),
 	    ((session->gpsdata.fix.longitude > 0) ? 'E' : 'W'),
-	    session->gpsdata.fix.speed * MPS_TO_KNOTS,
-	    session->gpsdata.fix.track,
+#define ZEROIZE(x)	(isnan(x)!=0 ? 0.0 : x)  
+	    ZEROIZE(session->gpsdata.fix.speed * MPS_TO_KNOTS),
+	    ZEROIZE(session->gpsdata.fix.track),
+#undef ZEROIZE
 	    tm.tm_mday,
-	    tm.tm_mon + 1,
-	    tm.tm_year % 100);
+	    tm.tm_mon,
+	    tm.tm_year);
     /*@ +usedef @*/
     nmea_add_checksum(bufp);
 }
@@ -502,7 +532,7 @@ static void gpsd_binary_dump(struct gps_device_t *session,
 {
     if ((session->gpsdata.set & LATLON_SET) != 0)
 	gpsd_binary_fix_dump(session, bufp+strlen(bufp), len-strlen(bufp));
-    if ((session->gpsdata.set & HDOP_SET) != 0)
+    if ((session->gpsdata.set & (DOP_SET | ERR_SET)) != 0)
 	gpsd_binary_quality_dump(session, bufp+strlen(bufp), len-strlen(bufp));
     if ((session->gpsdata.set & SATELLITE_SET) != 0)
 	gpsd_binary_satellite_dump(session,bufp+strlen(bufp),len-strlen(bufp));
@@ -525,7 +555,12 @@ void gpsd_error_model(struct gps_device_t *session,
      */
 #define UERE_NO_DGPS	8.0	/* meters, 95% confidence */
 #define UERE_WITH_DGPS	2.0	/* meters, 95% confidence */
-    double uere = (session->gpsdata.status == STATUS_DGPS_FIX ? UERE_WITH_DGPS : UERE_NO_DGPS);
+    double uere;
+
+    if (NULL == session)
+	return;
+
+    uere = (session->gpsdata.status == STATUS_DGPS_FIX ? UERE_WITH_DGPS : UERE_NO_DGPS);
 
     /*
      * Field reports match the theoretical prediction that
@@ -538,29 +573,33 @@ void gpsd_error_model(struct gps_device_t *session,
     /* Other error computations depend on having a valid fix */
     if (fix->mode >= MODE_2D) {
 	if (isnan(fix->eph)!=0 && finite(session->gpsdata.hdop)!=0)
-	    fix->eph = session->gpsdata.hdop * uere;
-	else
-	    fix->eph = NAN;
-	if ((fix->mode >= MODE_3D) 
-	    	&& isnan(fix->epv)!=0 && finite(session->gpsdata.vdop)!=0)
+		fix->eph = session->gpsdata.hdop * uere;
+
+	if ((fix->mode >= MODE_3D)
+		&& isnan(fix->epv)!=0 && finite(session->gpsdata.vdop)!=0)
 	    fix->epv = session->gpsdata.vdop * uere;
-	else
-	    fix->epv = NAN;
-	if (isnan(session->gpsdata.epe)!=0 && finite(session->gpsdata.vdop)!=0)
+
+	if (isnan(session->gpsdata.epe)!=0 && finite(session->gpsdata.pdop)!=0)
 	    session->gpsdata.epe = session->gpsdata.pdop * uere;
 	else
 	    session->gpsdata.epe = NAN;
+
 	/*
 	 * If we have a current fix and an old fix, and the packet handler 
 	 * didn't set the speed error and climb error members itself, 
 	 * try to compute them now.
 	 */
-	if (isnan(fix->eps)!=0 && fix->time > oldfix->time) {
-	    if (oldfix->mode > MODE_NO_FIX && fix->mode > MODE_NO_FIX) {
+	if (isnan(fix->eps)!=0)
+	{
+	    if (oldfix->mode > MODE_NO_FIX && fix->mode > MODE_NO_FIX
+			&& isnan(oldfix->eph)==0 && isnan(oldfix->eph)==0 
+			&& isnan(oldfix->time)==0 && isnan(oldfix->time)==0 
+			&& fix->time > oldfix->time) {
 		double t = fix->time-oldfix->time;
 		double e = oldfix->eph + fix->eph;
 		fix->eps = e/t;
-	    }
+	    } else
+		fix->eps = NAN;
 	}
 	if ((fix->mode >= MODE_3D) 
 	    	&& isnan(fix->epc)!=0 && fix->time > oldfix->time) {
@@ -571,7 +610,7 @@ void gpsd_error_model(struct gps_device_t *session,
 		fix->epc = e/t;
 	    }
 	    /*
-	     * We compute a track error estinate solely from the
+	     * We compute a track error estimate solely from the
 	     * position of this fix and the last one.  The maximum
 	     * track error, as seen from the position of last fix, is
 	     * the angle subtended by the two most extreme possible
@@ -658,16 +697,26 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 		(void)gpsd_switch_driver(session, "Zodiac binary");
 		break;
 #endif /* ZODIAC_ENABLE */
+#ifdef UBX_ENABLE
+	    case UBX_PACKET:
+		(void)gpsd_switch_driver(session, "uBlox UBX");
+		break;
+#endif /* UBX_ENABLE */
+#ifdef NAVCOM_ENABLE
+ 	    case NAVCOM_PACKET:
+         	(void)gpsd_switch_driver(session, "Navcom binary");
+         	break;
+#endif /* NAVCOM_ENABLE */
 #ifdef EVERMORE_ENABLE
 	    case EVERMORE_PACKET:
 		(void)gpsd_switch_driver(session, "EverMore binary");
 		break;
 #endif /* EVERMORE_ENABLE */
-#ifdef ITALK_ENABLE
+#ifdef ITRAX_ENABLE
 	    case ITALK_PACKET:
 		(void)gpsd_switch_driver(session, "iTalk binary");
 		break;
-#endif /* ITALK_ENABLE */
+#endif /* ITRAX_ENABLE */
 #ifdef RTCM104_ENABLE
 	    case RTCM_PACKET:
 		(void)gpsd_switch_driver(session, "RTCM104");
@@ -715,7 +764,7 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 
 	/*
 	 * Compute fix-quality data from the satellite positions.
-	 * This may be overridden by DOPs reported from the packet we just got.
+	 * These will not overwrite DOPs reported from the packet we just got.
 	 */
 	if (session->gpsdata.fix.mode > MODE_NO_FIX 
 		    && (session->gpsdata.set & SATELLITE_SET) != 0
@@ -725,8 +774,19 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	}
 	session->gpsdata.set = ONLINE_SET | dopmask | received;
 
-	/* count good fixes */
-	if (session->gpsdata.status > STATUS_NO_FIX) 
+	/*
+	 * Count good fixes. We used to check 
+	 *	session->gpsdata.status > STATUS_NO_FIX
+	 * here, but that wasn't quite right.  That tells us whether
+	 * we think we have a valid fix for the current cycle, but remains
+	 * true while following non-fix packets are received.  What we
+	 * really want to know is whether the last packet received was a
+	 * fix package AND held a valid fix. We must ignore non-fix packages
+	 * AND packages which have fix data but are flagged as invalid. Some
+	 * devices output fix packets on a regular basis, even when unable
+	 * to derive a good fix. Such packets should set STATUS_NO_FIX.
+	 */
+	if ((session->gpsdata.set & LATLON_SET )!=0 && session->gpsdata.status > STATUS_NO_FIX)
 	    session->context->fixcnt++;
 
 	session->gpsdata.d_decode_time = timestamp();
