@@ -1,4 +1,4 @@
-/* $Id: ntpshm.c 3771 2006-11-02 05:15:20Z esr $ */
+/* $Id: ntpshm.c 4438 2007-10-18 16:19:21Z mick_durkin $ */
 /* 
  * ntpshm.c - put time information in SHM segment for xntpd
  * struct shmTime and getShmTime from file in the xntp distribution:
@@ -41,7 +41,7 @@ struct shmTime {
     time_t clockTimeStampSec;
     int    clockTimeStampUSec;
     time_t receiveTimeStampSec;
-    long   receiveTimeStampUSec;
+    int    receiveTimeStampUSec;
     int    leap;
     int    precision;
     int    nsamples;
@@ -132,7 +132,7 @@ int ntpshm_put(struct gps_device_t *session, double fixtime)
     shmTime->clockTimeStampSec = (time_t)seconds;
     shmTime->clockTimeStampUSec = (int)microseconds;
     shmTime->receiveTimeStampSec = (time_t)tv.tv_sec;
-    shmTime->receiveTimeStampUSec = tv.tv_usec;
+    shmTime->receiveTimeStampUSec = (int)tv.tv_usec;
     /* setting the precision here does not seem to help anything, too
        hard to calculate properly anyway.  Let ntpd figure it out.
        Any NMEA will be about -1 or -2. 
@@ -195,7 +195,7 @@ int ntpshm_pps(struct gps_device_t *session, struct timeval *tv)
     shmTimeP->clockTimeStampSec = seconds;
     shmTimeP->clockTimeStampUSec = 0;
     shmTimeP->receiveTimeStampSec = (time_t)tv->tv_sec;
-    shmTimeP->receiveTimeStampUSec = tv->tv_usec;
+    shmTimeP->receiveTimeStampUSec = (int)tv->tv_usec;
     shmTimeP->precision = offset != 0 ? (int)(ceil(log(offset) / M_LN2)) : -20;
     shmTimeP->count++;
     shmTimeP->valid = 1;
