@@ -1,4 +1,4 @@
-/* $Id: packet.c 4103 2006-12-07 15:46:44Z esr $ */
+/* $Id: packet.c 4644 2007-12-28 03:13:44Z esr $ */
 /****************************************************************************
 
 NAME:
@@ -8,19 +8,19 @@ DESCRIPTION:
 
 Initial conditions of the problem:
 
-1. We have a file descriptor open for (possibly non-blocking) read. The device 
-   on the other end is sending packets at us.  
+1. We have a file descriptor open for (possibly non-blocking) read. The device
+   on the other end is sending packets at us.
 
 2. It may require more than one read to gather a packet.  Reads may span packet
    boundaries.
-  
+
 3. There may be leading garbage before the first packet.  After the first
    start-of-packet, the input should be well-formed.
 
 The problem: how do we recognize which kind of packet we're getting?
 
-No need to handle Garmin USB binary, we know that type by the fact we're 
-connected to the Garmin kernel driver.  But we need to be able to tell the 
+No need to handle Garmin USB binary, we know that type by the fact we're
+connected to the Garmin kernel driver.  But we need to be able to tell the
 others apart and distinguish them from baud barf.
 
 ***************************************************************************/
@@ -33,7 +33,7 @@ others apart and distinguish them from baud barf.
 #include "gpsd_config.h"
 #include "gpsd.h"
 
-/* 
+/*
  * The packet-recognition state machine.  It can be fooled by garbage
  * that looks like the head of a binary packet followed by a NMEA
  * packet; in that case it won't reset until it notices that the
@@ -70,11 +70,11 @@ enum {
 #define STX	0x02
 #define ETX	0x03
 
-static void nextstate(struct gps_packet_t *lexer, 
+static void nextstate(struct gps_packet_t *lexer,
 		      unsigned char c)
 {
 #ifdef RTCM104_ENABLE
-    enum isgpsstat_t	isgpsstat;    
+    enum isgpsstat_t	isgpsstat;
 #endif /* RTCM104_ENABLE */
 /*@ +charint */
     switch(lexer->state)
@@ -89,27 +89,31 @@ static void nextstate(struct gps_packet_t *lexer,
 	    lexer->state = NMEA_DOLLAR;
 	    break;
 	}
+	if (c == '!') {
+	    lexer->state = NMEA_BANG;
+	    break;
+	}
 #endif /* NMEA_ENABLE */
-#ifdef TNT_ENABLE
-        if (c == '@') {
+#if defined(TNT_ENABLE) || defined(GARMINTXT_ENABLE)
+	if (c == '@') {
 	    lexer->state = TNT_LEADER;
 	    break;
 	}
 #endif
 #ifdef SIRF_ENABLE
-        if (c == 0xa0) {
+	if (c == 0xa0) {
 	    lexer->state = SIRF_LEADER_1;
 	    break;
 	}
 #endif /* SIRF_ENABLE */
 #if defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) || defined(GARMIN_ENABLE)
-        if (c == DLE) {
+	if (c == DLE) {
 	    lexer->state = DLE_LEADER;
 	    break;
 	}
 #endif /* TSIP_ENABLE || EVERMORE_ENABLE || GARMIN_ENABLE */
 #ifdef TRIPMATE_ENABLE
-        if (c == 'A') {
+	if (c == 'A') {
 #ifdef RTCM104_ENABLE
 	    if (rtcm_decode(lexer, c) == ISGPS_MESSAGE) {
 		lexer->state = RTCM_RECOGNIZED;
@@ -121,7 +125,7 @@ static void nextstate(struct gps_packet_t *lexer,
 	}
 #endif /* TRIPMATE_ENABLE */
 #ifdef EARTHMATE_ENABLE
-        if (c == 'E') {
+	if (c == 'E') {
 #ifdef RTCM104_ENABLE
 	    if (rtcm_decode(lexer, c) == ISGPS_MESSAGE) {
 		lexer->state = RTCM_RECOGNIZED;
@@ -138,12 +142,24 @@ static void nextstate(struct gps_packet_t *lexer,
 	    break;
 	}
 #endif /* ZODIAC_ENABLE */
-#ifdef ITALK_ENABLE
+#ifdef UBX_ENABLE
+	if (c == 0xb5) {
+	    lexer->state = UBX_LEADER_1;
+	    break;
+	}
+#endif /* UBX_ENABLE */
+#ifdef ITRAX_ENABLE
 	if (c == '<') {
 	    lexer->state = ITALK_LEADER_1;
 	    break;
 	}
-#endif /* ITALK_ENABLE */
+#endif /* ITRAX_ENABLE */
+#ifdef NAVCOM_ENABLE
+	if (c == 0x02) {
+	    lexer->state = NAVCOM_LEADER_1;
+	    break;
+	}
+#endif /* NAVCOM_ENABLE */
 #ifdef RTCM104_ENABLE
 	if ((isgpsstat = rtcm_decode(lexer, c)) == ISGPS_SYNC) {
 	    lexer->state = RTCM_SYNC_STATE;
@@ -160,13 +176,13 @@ static void nextstate(struct gps_packet_t *lexer,
 	    lexer->state = COMMENT_RECOGNIZED;
 	else if (!isprint(c))
 	    lexer->state = GROUND_STATE;
-	break;	    
+	break;
 #ifdef NMEA_ENABLE
     case NMEA_DOLLAR:
 	if (c == 'G')
 	    lexer->state = NMEA_PUB_LEAD;
 	else if (c == 'P')	/* vendor sentence */
-	    lexer->state = NMEA_LEADER_END;
+	    lexer->state = NMEA_VENDOR_LEAD;
 	else if (c =='I')	/* Seatalk */
 	    lexer->state = SEATALK_LEAD_1;
 	else if (c =='A')	/* SiRF Ack */
@@ -180,10 +196,34 @@ static void nextstate(struct gps_packet_t *lexer,
 	else
 	    lexer->state = GROUND_STATE;
 	break;
-#ifdef TNT_ENABLE
+    case NMEA_VENDOR_LEAD:
+	if (isalpha(c))
+	    lexer->state = NMEA_LEADER_END;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case NMEA_BANG:
+	if (c == 'A')
+	    lexer->state = AIS_LEAD_1;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case AIS_LEAD_1:
+	if (c == 'I')
+	    lexer->state = AIS_LEAD_2;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case AIS_LEAD_2:
+	if (isalpha(c))
+	    lexer->state = NMEA_LEADER_END;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+#if defined(TNT_ENABLE) || defined(GARMINTXT_ENABLE)
     case TNT_LEADER:
-          lexer->state = NMEA_LEADER_END;
-        break;
+	  lexer->state = NMEA_LEADER_END;
+	break;
 #endif
     case NMEA_LEADER_END:
 	if (c == '\r')
@@ -206,6 +246,8 @@ static void nextstate(struct gps_packet_t *lexer,
     case NMEA_RECOGNIZED:
 	if (c == '$')
 	    lexer->state = NMEA_DOLLAR;
+	else if (c == '!')
+	    lexer->state = NMEA_BANG;
 	else
 	    lexer->state = GROUND_STATE;
 	break;
@@ -337,11 +379,13 @@ static void nextstate(struct gps_packet_t *lexer,
 	    lexer->state = NMEA_RECOGNIZED;
 	} else
 	    lexer->state = GROUND_STATE;
-	break; 
+	break;
 #endif /* EARTHMATE_ENABLE */
     case SIRF_ACK_LEAD_1:
 	if (c == 'c')
 	    lexer->state = SIRF_ACK_LEAD_2;
+	else if (c == 'I')
+	    lexer->state = AIS_LEAD_2;
 	else
 	    lexer->state = GROUND_STATE;
 	break;
@@ -387,7 +431,7 @@ static void nextstate(struct gps_packet_t *lexer,
 	    lexer->state = GROUND_STATE;
 	break;
     case SIRF_RECOGNIZED:
-        if (c == 0xa0)
+	if (c == 0xa0)
 	    lexer->state = SIRF_LEADER_1;
 	else
 	    lexer->state = GROUND_STATE;
@@ -398,17 +442,74 @@ static void nextstate(struct gps_packet_t *lexer,
 #ifdef EVERMORE_ENABLE
 	if (c == STX)
 	    lexer->state = EVERMORE_LEADER_2;
-	else
 #endif /* EVERMORE_ENABLE */
-#if defined(TSIP_ENABLE) || defined(GARMIN_ENABLE)
+#if defined(TSIP_ENABLE) || defined(GARMIN_ENABLE) || defined(NAVCOM_ENABLE)
 	/* garmin is special case of TSIP */
 	/* check last because there's no checksum */
 	if (c >= 0x13)
 	    lexer->state = TSIP_PAYLOAD;
-	else
-#endif /* TSIP_ENABLE */
+	if (c == DLE)
 	    lexer->state = GROUND_STATE;
 	break;
+#endif /* TSIP_ENABLE */
+#ifdef NAVCOM_ENABLE
+    case NAVCOM_LEADER_1:
+	if (c == 0x99)
+	    lexer->state = NAVCOM_LEADER_2;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case NAVCOM_LEADER_2:
+	if (c == 0x66)
+	    lexer->state = NAVCOM_LEADER_3;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case NAVCOM_LEADER_3:
+	lexer->state = NAVCOM_ID;
+	break;
+    case NAVCOM_ID:
+	lexer->length = (size_t)c - 4;
+	lexer->state = NAVCOM_LENGTH_1;
+	break;
+    case NAVCOM_LENGTH_1:
+	lexer->length += (c << 8);
+	lexer->state = NAVCOM_LENGTH_2;
+	break;
+    case NAVCOM_LENGTH_2:
+	if (--lexer->length == 0)
+	    lexer->state = NAVCOM_PAYLOAD;
+	break;
+    case NAVCOM_PAYLOAD:
+	{
+	    unsigned int n;
+	    unsigned char csum = lexer->inbuffer[3];
+	    for(n=4; (unsigned char *)(lexer->inbuffer + n) < lexer->inbufptr - 1; n++)
+	    csum ^= lexer->inbuffer[n];
+	    if(csum != c) {
+	    gpsd_report(LOG_INF, "Navcom packet type 0x%hx bad checksum 0x%hx, expecting 0x%hx\n",
+		    lexer->inbuffer[3], csum, c);
+	    gpsd_report(LOG_RAW, "Navcom packet dump: %s\n",
+			gpsd_hexdump(lexer->inbuffer, lexer->inbuflen));
+	    lexer->state = GROUND_STATE;
+	    break;
+	    }
+	}
+	lexer->state = NAVCOM_CSUM;
+	break;
+    case NAVCOM_CSUM:
+	if (c == 0x03)
+	    lexer->state = NAVCOM_RECOGNIZED;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case NAVCOM_RECOGNIZED:
+	if (c == 0x02)
+	    lexer->state = NAVCOM_LEADER_1;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+#endif /* NAVCOM_ENABLE */
 #endif /* TSIP_ENABLE || EVERMORE_ENABLE || GARMIN_ENABLE */
 #ifdef ZODIAC_ENABLE
     case ZODIAC_EXPECTED:
@@ -453,13 +554,15 @@ static void nextstate(struct gps_packet_t *lexer,
 	    short sum = getword(0) + getword(1) + getword(2) + getword(3);
 	    sum *= -1;
 	    if (sum != getword(4)) {
-		gpsd_report(LOG_IO, "Zodiac Header checksum 0x%hx expecting 0x%hx\n", 
-		       sum, getword(4));
+		gpsd_report(LOG_IO,
+		    "Zodiac Header checksum 0x%hx expecting 0x%hx\n",
+		    sum, getword(4));
 		lexer->state = GROUND_STATE;
 		break;
 	    }
 	}
-	gpsd_report(LOG_RAW+1,"Zodiac header id=%hd len=%hd flags=%hx\n", getword(1), getword(2), getword(3));
+	gpsd_report(LOG_RAW+1,"Zodiac header id=%hd len=%hd flags=%hx\n",
+	    getword(1), getword(2), getword(3));
  #undef getword
 	if (lexer->length == 0) {
 	    lexer->state = ZODIAC_RECOGNIZED;
@@ -478,6 +581,48 @@ static void nextstate(struct gps_packet_t *lexer,
 	    lexer->state = ZODIAC_RECOGNIZED;
 	break;
 #endif /* ZODIAC_ENABLE */
+#ifdef UBX_ENABLE
+    case UBX_LEADER_1:
+	if (c == 0x62)
+	    lexer->state = UBX_LEADER_2;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case UBX_LEADER_2:
+	lexer->state = UBX_CLASS_ID;
+	break;
+    case UBX_CLASS_ID:
+	lexer->state = UBX_MESSAGE_ID;
+	break;
+    case UBX_MESSAGE_ID:
+	lexer->length = (size_t)c;
+	lexer->state = UBX_LENGTH_1;
+	break;
+    case UBX_LENGTH_1:
+	lexer->length += (c << 8);
+	if (lexer->length <= MAX_PACKET_LENGTH)
+	    lexer->state = UBX_LENGTH_2;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case UBX_LENGTH_2:
+	lexer->state = UBX_PAYLOAD;
+	break;
+    case UBX_PAYLOAD:
+	if (--lexer->length == 0)
+	    lexer->state = UBX_CHECKSUM_A;
+	/* else stay in payload state */
+	break;
+    case UBX_CHECKSUM_A:
+	lexer->state = UBX_RECOGNIZED;
+	break;
+    case UBX_RECOGNIZED:
+	if (c == 0xb5)
+	    lexer->state = UBX_LEADER_1;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+#endif /* UBX_ENABLE */
 #ifdef EVERMORE_ENABLE
     case EVERMORE_LEADER_1:
 	if (c == STX)
@@ -499,40 +644,44 @@ static void nextstate(struct gps_packet_t *lexer,
 	    lexer->state = GROUND_STATE;
 	break;
     case EVERMORE_PAYLOAD_DLE:
-        switch (c) {
-           case DLE: lexer->state = EVERMORE_PAYLOAD; break;
-           case ETX: lexer->state = EVERMORE_RECOGNIZED; break;
-           default: lexer->state = GROUND_STATE;
-        }
+	switch (c) {
+	   case DLE: lexer->state = EVERMORE_PAYLOAD; break;
+	   case ETX: lexer->state = EVERMORE_RECOGNIZED; break;
+	   default: lexer->state = GROUND_STATE;
+	}
     break;
     case EVERMORE_RECOGNIZED:
-        if (c == DLE)
+	if (c == DLE)
 	    lexer->state = EVERMORE_LEADER_1;
 	else
 	    lexer->state = GROUND_STATE;
 	break;
 #endif /* EVERMORE_ENABLE */
-#ifdef ITALK_ENABLE
+#ifdef ITRAX_ENABLE
     case ITALK_LEADER_1:
-        if (c == '!')
+	if (c == '!')
 	    lexer->state = ITALK_LEADER_2;
 	else
 	    lexer->state = GROUND_STATE;
 	break;
     case ITALK_LEADER_2:
-	lexer->length = (size_t)(c & 0xff);
+	lexer->length = (size_t)(lexer->inbuffer[6] & 0xff);
 	lexer->state = ITALK_LENGTH;
 	break;
     case ITALK_LENGTH:
 	lexer->length += 1;	/* fix number of words in payload */
 	lexer->length *= 2;	/* convert to number of bytes */
+	lexer->length += 3;	/* add trailer length */
 	lexer->state = ITALK_PAYLOAD;
 	break;
     case ITALK_PAYLOAD:
 	/* lookahead for "<!" because sometimes packets are short but valid */
-	if ((c == '>') && (lexer->inbufptr[0] == '<') && (lexer->inbufptr[1] == '!'))
+	if ((c == '>') && (lexer->inbufptr[0] == '<') &&
+	    (lexer->inbufptr[1] == '!')){
 	    lexer->state = ITALK_RECOGNIZED;
-	else if (--lexer->length == 0)
+	    gpsd_report(LOG_PROG, "ITALK: trying to process runt packet\n");
+	    break;
+	} else if (--lexer->length == 0)
 	    lexer->state = ITALK_DELIVERED;
 	break;
     case ITALK_DELIVERED:
@@ -542,15 +691,15 @@ static void nextstate(struct gps_packet_t *lexer,
 	    lexer->state = GROUND_STATE;
 	break;
     case ITALK_RECOGNIZED:
-        if (c == '<')
+	if (c == '<')
 	    lexer->state = ITALK_LEADER_1;
 	else
 	    lexer->state = GROUND_STATE;
 	break;
-#endif /* ITALK_ENABLE */
+#endif /* ITRAX_ENABLE */
 #ifdef TSIP_ENABLE
     case TSIP_LEADER:
-        /* unused case */
+	/* unused case */
 	if (c >= 0x13)
 	    lexer->state = TSIP_PAYLOAD;
 	else
@@ -575,12 +724,12 @@ static void nextstate(struct gps_packet_t *lexer,
 	}
 	break;
     case TSIP_RECOGNIZED:
-        if (c == DLE)
+	if (c == DLE)
 	    /*
 	     * Don't go to TSIP_LEADER state -- TSIP packets aren't
 	     * checksummed, so false positives are easy.  We might be
 	     * looking at another DLE-stuffed protocol like EverMore
-             * or Garmin streaming binary.
+	     * or Garmin streaming binary.
 	     */
 	    lexer->state = DLE_LEADER;
 	else
@@ -687,9 +836,9 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 	};
 	nextstate(lexer, c);
 	gpsd_report(LOG_RAW+2, "%08ld: character '%c' [%02x], new state: %s\n",
-		    lexer->char_counter, 
-		    (isprint(c)?c:'.'), 
-		    c, 
+		    lexer->char_counter,
+		    (isprint(c)?c:'.'),
+		    c,
 		    state_table[lexer->state]);
 	lexer->char_counter++;
 
@@ -720,7 +869,7 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 	    else
 		lexer->state = GROUND_STATE;
 	    packet_discard(lexer);
-            break;
+	    break;
 	}
 #endif /* NMEA_ENABLE */
 #ifdef SIRF_ENABLE
@@ -736,22 +885,27 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 	    else
 		lexer->state = GROUND_STATE;
 	    packet_discard(lexer);
-            break;
+	    break;
 	}
 #endif /* SIRF_ENABLE */
 #if defined(TSIP_ENABLE) || defined(GARMIN_ENABLE)
 	else if (lexer->state == TSIP_RECOGNIZED) {
-            size_t packetlen = lexer->inbufptr - lexer->inbuffer;
+	    size_t packetlen = lexer->inbufptr - lexer->inbuffer;
 	    if ( packetlen < 5) {
 		lexer->state = GROUND_STATE;
-            } else {
+	    } else {
 		unsigned int pkt_id, len;
 		size_t n;
 #ifdef GARMIN_ENABLE
 		unsigned int ch, chksum;
 		n = 0;
 		/*@ +charint */
-		if (lexer->inbuffer[n++] != DLE) 
+#ifdef TSIP_ENABLE
+		/* shortcut garmin */
+		if (TSIP_PACKET == lexer->type)
+			goto not_garmin;
+#endif /* TSIP_ENABLE */
+		if (lexer->inbuffer[n++] != DLE)
 		    goto not_garmin;
 		pkt_id = lexer->inbuffer[n++]; /* packet ID */
 		len = lexer->inbuffer[n++];
@@ -776,7 +930,7 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 		}
 		if (lexer->inbuffer[n++] != DLE)
 		    goto not_garmin;
-		if (lexer->inbuffer[n++] != ETX) 
+		if (lexer->inbuffer[n++] != ETX)
 		    goto not_garmin;
 		/*@ +charint */
 		chksum &= 0xff;
@@ -793,10 +947,11 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 		packet_discard(lexer);
 		break;
 	    not_garmin:;
-	        gpsd_report(LOG_RAW+1,"Not a Garmin packet\n");
+		gpsd_report(LOG_RAW+1,"Not a Garmin packet\n");
 #endif /* GARMIN_ENABLE */
 #ifdef TSIP_ENABLE
 		/* check for some common TSIP packet types:
+		 * 0x13, TSIP Parsing Error Notification
 		 * 0x41, GPS time, data length 10
 		 * 0x42, Single Precision Fix, data length 16
 		 * 0x43, Velocity Fix, data length 20
@@ -810,56 +965,69 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 		 * 0x82, Differential Position Fix Mode, data length 1
 		 * 0x83, Double Precision XYZ, data length 36
 		 * 0x84, Double Precision LLA, data length 36
+		 * 0xbb, GPS Navigation Configuration
+		 * 0xbc, Receiver Port Configuration
 		 *
 		 * <DLE>[pkt id] [data] <DLE><ETX>
 		 */
 		/*@ +charint @*/
 		pkt_id = lexer->inbuffer[1]; /* packet ID */
-		if ((0x41 > pkt_id) || (0x8f < pkt_id)) {
-		    gpsd_report(LOG_RAW+1, "Packet ID out of range for TSIP\n");
+		if (!((0x13 == pkt_id) || (0xbb == pkt_id) || (0xbc == pkt_id)) &&
+		    ((0x41 > pkt_id) || (0x8f < pkt_id))) {
+		    gpsd_report(LOG_IO, "Packet ID 0x%02x out of range for TSIP\n", pkt_id);
 		    goto not_tsip;
 		}
 		/*@ -ifempty */
-		if ((0x41 == pkt_id) && (0x0c == packetlen))
+		if ((0x13 == pkt_id) && (0x01 <= packetlen))
 		    /* pass */;
-		else if ((0x42 == pkt_id) && (0x12 == packetlen ))
+		else if ((0x41 == pkt_id) && (0x0e == packetlen))
 		    /* pass */;
-		else if ((0x43 == pkt_id) && (0x16 == packetlen))
+		else if ((0x42 == pkt_id) && (0x14 == packetlen ))
 		    /* pass */;
-		else if ((0x45 == pkt_id) && (0x0c == packetlen))
+		else if ((0x43 == pkt_id) && (0x18 == packetlen))
 		    /* pass */;
-		else if ((0x46 == pkt_id) && (0x04 == packetlen))
+		else if ((0x45 == pkt_id) && (0x0e == packetlen))
 		    /* pass */;
-		else if ((0x4a == pkt_id) && (0x16 == packetlen))
+		else if ((0x46 == pkt_id) && (0x06 == packetlen))
 		    /* pass */;
-		else if ((0x4b == pkt_id) && (0x05 == packetlen))
+		else if ((0x4a == pkt_id) && (0x18 == packetlen))
 		    /* pass */;
-		else if ((0x56 == pkt_id) && (0x16 == packetlen))
+		else if ((0x4b == pkt_id) && (0x07 == packetlen))
 		    /* pass */;
-		else if ((0x5c == pkt_id) && (0x1a == packetlen))
+		else if ((0x55 == pkt_id) && (0x08 == packetlen))
 		    /* pass */;
-		else if ((0x6d == pkt_id) && ((0x12 <= packetlen) && (0x1e >= packetlen) ))
+		else if ((0x56 == pkt_id) && (0x18 == packetlen))
 		    /* pass */;
-		else if ((0x82 == pkt_id) && (0x03 == packetlen))
+		else if ((0x5c == pkt_id) && ((0x1c <= packetlen) && (0x1e >= packetlen)))
+		    /* pass */;
+		else if ((0x6d == pkt_id) && ((0x14 <= packetlen) && (0x20 >= packetlen)))
+		    /* pass */;
+		else if ((0x82 == pkt_id) && (0x05 == packetlen))
+		    /* pass */;
+		else if ((0x84 == pkt_id) && ((0x28 <= packetlen) && (0x29 >= packetlen)))
+		    /* pass */;
+		else if ((0x8e == pkt_id))
 		    /* pass */;
 		else if ((0x8f == pkt_id))
 		    /* pass */;
+		else if ((0xbb == pkt_id) && (0x2c == packetlen))
+		    /* pass */;
 		else {
 		    gpsd_report(LOG_IO,
 			"TSIP REJECT pkt_id = %#02x, packetlen= %#02x\n",
-			pkt_id, packetlen); 
+			pkt_id, packetlen);
 		    goto not_tsip;
 		}
 		/* Debug */
 		gpsd_report(LOG_RAW,
 		    "TSIP pkt_id = %#02x, packetlen= %#02x\n",
-		    pkt_id, packetlen); 
+		    pkt_id, packetlen);
 		/*@ -charint +ifempty @*/
 		packet_accept(lexer, TSIP_PACKET);
 		packet_discard(lexer);
 		break;
 	    not_tsip:
-	        gpsd_report(LOG_RAW+1,"Not a TSIP packet\n");
+		gpsd_report(LOG_RAW+1,"Not a TSIP packet\n");
 		/*
 		 * More attempts to recognize ambiguous TSIP-like
 		 * packet types could go here.
@@ -887,15 +1055,47 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 		lexer->state = GROUND_STATE;
 	    }
 	    packet_discard(lexer);
-            break;
+	    break;
 	}
 #endif /* ZODIAC_ENABLE */
+#ifdef UBX_ENABLE
+	else if (lexer->state == UBX_RECOGNIZED) {
+	    /* UBX use a TCP like checksum */
+	    int n, len;
+	    unsigned char ck_a = (unsigned char)0;
+	    unsigned char ck_b = (unsigned char)0;
+	    len = lexer->inbufptr - lexer->inbuffer;
+	    gpsd_report(LOG_IO, "UBX: len %d\n", len);
+	    for (n = 2; n < (len-2); n++) {
+		ck_a += lexer->inbuffer[n];
+		ck_b += ck_a;
+	    }
+	    if (ck_a == lexer->inbuffer[len-2] &&
+		ck_b == lexer->inbuffer[len-1])
+		packet_accept(lexer, UBX_PACKET);
+	    else {
+		gpsd_report(LOG_IO,
+		    "UBX checksum 0x%02hhx%02hhx over length %hd,"\
+		    " expecting 0x%02hhx%02hhx (type 0x%02hhx%02hhx)\n",
+			ck_a,
+			ck_b,
+			len,
+			lexer->inbuffer[len-2],
+			lexer->inbuffer[len-1],
+			lexer->inbuffer[2],
+			lexer->inbuffer[3]);
+		lexer->state = GROUND_STATE;
+	    }
+	    packet_discard(lexer);
+	    break;
+	}
+#endif /* UBX_ENABLE */
 #ifdef EVERMORE_ENABLE
 	else if (lexer->state == EVERMORE_RECOGNIZED) {
 	    unsigned int n, crc, checksum, len;
 	    n = 0;
 	    /*@ +charint */
-	    if (lexer->inbuffer[n++] != DLE) 
+	    if (lexer->inbuffer[n++] != DLE)
 		goto not_evermore;
 	    if (lexer->inbuffer[n++] != STX)
 		goto not_evermore;
@@ -924,8 +1124,8 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 		goto not_evermore;
 	    crc &= 0xff;
 	    if (crc != checksum) {
-		gpsd_report(LOG_IO, 
-			    "EverMore checksum failed: %02x != %02x\n", 
+		gpsd_report(LOG_IO,
+			    "EverMore checksum failed: %02x != %02x\n",
 			    crc, checksum);
 		goto not_evermore;
 	    }
@@ -936,40 +1136,66 @@ ssize_t packet_parse(struct gps_packet_t *lexer, size_t fix)
 	not_evermore:
 	    lexer->state = GROUND_STATE;
 	    packet_discard(lexer);
-            break;
+	    break;
 	}
 #endif /* EVERMORE_ENABLE */
-#ifdef ITALK_ENABLE
+/* XXX CSK */
+#ifdef ITRAX_ENABLE
+#define getib(j) ((u_int8_t)lexer->inbuffer[(j)])
+#define getiw(i) ((u_int16_t)(((u_int16_t)getib((i)+1) << 8) | (u_int16_t)getib((i))))
+
 	else if (lexer->state == ITALK_RECOGNIZED) {
-	    u_int16_t len, n, sum;
-	    len = (unsigned short)(lexer->length / 2 - 1);
-	    /*
-	     * Skip first 9 words so we compute checksum only over data
-	     * portion of packet.
-	     */
-	    for (n = sum = 0; n < (unsigned short)(len - 9); n++)
-		sum += getword(9 + n);
-	    if (len == 0 || sum == (u_int16_t)getword(len+1)) {
-		gpsd_report(LOG_RAW, "italk checksum ok\n");
+	    volatile u_int16_t len, n, csum, xsum, tmpw;
+	    volatile u_int32_t tmpdw;
+
+	    /* number of words */
+	    len = (unsigned short)(lexer->inbuffer[6] &0xff);
+
+	    /*@ -type @*/
+	    /* initialize all my registers */
+	    csum = tmpw = tmpdw = 0;
+	    /* expected checksum */
+	    xsum = getiw(7+2*len);
+
+	    for (n = 0; n < len; n++){
+		tmpw = getiw(7 + 2*n);
+		tmpdw = (csum + 1) * (tmpw + n);
+		csum ^= (tmpdw & 0xffff) ^ ((tmpdw >>16) & 0xffff);
+	    }
+	    /*@ +type @*/
+	    if (len == 0 || csum == xsum)
 		packet_accept(lexer, ITALK_PACKET);
-	    } else {
-		gpsd_report(LOG_RAW, "italk checksum failed\n");
+	    else {
+		gpsd_report(LOG_IO,
+		    "ITALK: checksum failed - "
+		    "type 0x%02x expected 0x%04x got 0x%04x\n",
+		    lexer->inbuffer[4], xsum, csum);
 		lexer->state = GROUND_STATE;
 	    }
 	    packet_discard(lexer);
-            break;
+	    break;
 	}
-#endif /* ITALK_ENABLE */
+#undef getiw
+#undef getib
+#endif /* ITRAX_ENABLE */
+#ifdef NAVCOM_ENABLE
+	else if (lexer->state == NAVCOM_RECOGNIZED) {
+	    /* By the time we got here we know checksum is OK */
+	    packet_accept(lexer, NAVCOM_PACKET);
+	    packet_discard(lexer);
+	    break;
+	}
+#endif /* NAVCOM_ENABLE */
 #ifdef RTCM104_ENABLE
 	else if (lexer->state == RTCM_RECOGNIZED) {
 	    /*
-	     * RTCM packets don't have checksums.  The six bits of parity 
+	     * RTCM packets don't have checksums.  The six bits of parity
 	     * per word and the preamble better be good enough.
 	     */
 	    packet_accept(lexer, RTCM_PACKET);
 	    lexer->state = RTCM_SYNC_STATE;
 	    packet_discard(lexer);
-            break;
+	    break;
 	}
 #endif /* RTCM104_ENABLE */
     } /* while */
@@ -992,14 +1218,14 @@ ssize_t packet_get(int fd, struct gps_packet_t *lexer)
 #endif /* STATEDEBUG */
     /*@ +modobserver @*/
     if (recvd == -1) {
-        if ((errno == EAGAIN) || (errno == EINTR)) {
+	if ((errno == EAGAIN) || (errno == EINTR)) {
 	    return 0;
-        } else {
+	} else {
 #ifdef STATEDEBUG
 	    gpsd_report(LOG_RAW+1, "errno: %s\n", strerror(errno));
 #endif /* STATEDEBUG */
 	    return BAD_PACKET;
-        }
+	}
     }
 
     if (recvd == 0)
