@@ -1,4 +1,4 @@
-/* $Id: packet_states.h 4049 2006-12-01 18:21:29Z esr $ */
+/* $Id: packet_states.h 4644 2007-12-28 03:13:44Z esr $ */
    GROUND_STATE,	/* we don't know what packet type to expect */
 
    COMMENT_BODY,	/* pound comment for a test load */
@@ -6,13 +6,17 @@
 
 #ifdef NMEA_ENABLE
    NMEA_DOLLAR,		/* we've seen first character of NMEA leader */
+   NMEA_BANG,		/* we've seen first character of an AIS message '!' */
    NMEA_PUB_LEAD,	/* seen second character of NMEA G leader */
+   NMEA_VENDOR_LEAD,	/* seen second character of NMEA P leader */
    NMEA_LEADER_END,	/* seen end char of NMEA leader, in body */
    NMEA_CR,	   	/* seen terminating \r of NMEA packet */
    NMEA_RECOGNIZED,	/* saw trailing \n of NMEA packet */
 
    SIRF_ACK_LEAD_1,	/* seen A of possible SiRF Ack */
    SIRF_ACK_LEAD_2,	/* seen c of possible SiRF Ack */
+   AIS_LEAD_1,		/* seen A of possible marine AIS message */
+   AIS_LEAD_2,		/* seen I of possible marine AIS message */
 
    SEATALK_LEAD_1,	/* SeaTalk/Garmin packet leader 'I' */
 #endif /* NMEA_ENABLE */
@@ -60,8 +64,9 @@
    ZODIAC_RECOGNIZED,	/* found end of the Zodiac packet */
 #endif /* ZODIAC_ENABLE */
 
-#ifdef TNT_ENABLE
+#if defined(TNT_ENABLE) || defined(GARMINTXT_ENABLE)
    TNT_LEADER,          /* saw True North status leader '@' */
+                        /* Garmin Simple Text starts with @ leader */
 #endif
 
 #ifdef EVERMORE_ENABLE
@@ -72,7 +77,7 @@
    EVERMORE_RECOGNIZED,	/* found end of EverMore packet */
 #endif /* EVERMORE_ENABLE */
 
-#ifdef ITALK_ENABLE
+#ifdef ITRAX_ENABLE
    ITALK_LEADER_1,	/* saw leading < of iTalk packet */
    ITALK_LEADER_2,	/* saw leading ! of iTalk packet */
    ITALK_LENGTH,	/* saw packet length */
@@ -80,7 +85,32 @@
    ITALK_DELIVERED,	/* seen end of payload */
    ITALK_TRAILER,	/* saw iTalk trailer byte */
    ITALK_RECOGNIZED,	/* found end of the iTalk packet */
-#endif /* ITALK_ENABLE */
+#endif /* ITRAX_ENABLE */
+
+#ifdef NAVCOM_ENABLE
+   NAVCOM_EXPECTED,	/* expecting Navcom packet */
+   NAVCOM_LEADER_1,	/* saw leading 0x02 */
+   NAVCOM_LEADER_2,	/* saw leading 0x99 */
+   NAVCOM_LEADER_3,	/* saw leading 0x66 */
+   NAVCOM_ID,   	/* saw message ID */
+   NAVCOM_LENGTH_1,	/* saw first byte of Navcom packet length */
+   NAVCOM_LENGTH_2,	/* saw second byte of Navcom packet length */
+   NAVCOM_PAYLOAD,	/* we're in a Navcom payload */
+   NAVCOM_CSUM, 	/* saw checksum */
+   NAVCOM_RECOGNIZED,	/* found end of the Navcom packet */
+#endif /* NAVCOM_ENABLE */
+
+#ifdef UBX_ENABLE
+   UBX_LEADER_1,        /* first constant leader byte found */
+   UBX_LEADER_2,        /* second constant leader byte found */
+   UBX_CLASS_ID,        /* classid read */
+   UBX_MESSAGE_ID,      /* message id read */
+   UBX_LENGTH_1,        /* first length byte read (le) */
+   UBX_LENGTH_2,        /* second length byte read (le) */
+   UBX_PAYLOAD,         /* payload eating */
+   UBX_CHECKSUM_A,      /* checksum A byte (tcp checksum) */
+   UBX_RECOGNIZED,      /* this is also UBX_CHECKSUM_B */
+#endif
 
 /*
  * Packet formats without checksums start here.  We list them last so
@@ -102,5 +132,4 @@
    RTCM_RECOGNIZED,	/* we have an RTCM packet */
 #endif /* RTCM104_ENABLE */
 
-
 /* end of packet_states.h */
