@@ -1,5 +1,6 @@
-/* $Id: serial.c 4080 2006-12-05 13:35:45Z esr $ */
+/* $Id: serial.c 4550 2007-12-13 04:14:09Z esr $ */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
@@ -26,6 +27,11 @@ void gpsd_tty_init(struct gps_device_t *session)
     /* mark GPS fd closed and its baud rate unknown */
     session->gpsdata.gps_fd = -1;
     session->saved_baud = -1;
+#ifdef NTPSHM_ENABLE
+    /* mark NTPD shared memory segments as unused */
+    session->shmindex = -1;
+    session->shmTimeP = -1;
+#endif /* NTPSHM_ENABLE */
 }
 
 #if defined(__CYGWIN__)
@@ -183,7 +189,7 @@ void gpsd_set_speed(struct gps_device_t *session,
      * If we don't know the device type, ship it every driver's wakeup
      * in hopes it will respond.
      */
-    if (isatty(session->gpsdata.gps_fd)!=0) {
+    if (isatty(session->gpsdata.gps_fd)!=0 && !session->context->readonly) {
 	struct gps_type_t **dp;
 	if (session->device_type == NULL) {
 	    for (dp = gpsd_drivers; *dp; dp++)
@@ -197,9 +203,20 @@ void gpsd_set_speed(struct gps_device_t *session,
 
 int gpsd_open(struct gps_device_t *session)
 {
-    gpsd_report(LOG_INF, "opening GPS data source at '%s'\n", session->gpsdata.gps_device);
-    if ((session->gpsdata.gps_fd = open(session->gpsdata.gps_device, O_RDWR|O_NONBLOCK|O_NOCTTY)) < 0) {
-	gpsd_report(LOG_ERROR, "read-write device open failed: %s - trying read-only\n", strerror(errno));
+    struct stat sb;
+    mode_t mode = (mode_t)O_RDWR;
+
+    /*@ -boolops @*/
+    if (session->context->readonly || ((stat(session->gpsdata.gps_device, &sb) != -1) && ((sb.st_mode & S_IFCHR) != S_IFCHR))){
+	mode = (mode_t)O_RDONLY;
+	gpsd_report(LOG_INF, "opening read-only GPS data source at '%s'\n", session->gpsdata.gps_device);
+    } else {
+	gpsd_report(LOG_INF, "opening GPS data source at '%s'\n", session->gpsdata.gps_device);
+    }
+    /*@ +boolops @*/
+
+    if ((session->gpsdata.gps_fd = open(session->gpsdata.gps_device, (int)(mode|O_NONBLOCK|O_NOCTTY))) < 0) {
+	gpsd_report(LOG_ERROR, "device open failed: %s - retrying read-only\n", strerror(errno));
 	if ((session->gpsdata.gps_fd = open(session->gpsdata.gps_device, O_RDONLY|O_NONBLOCK|O_NOCTTY)) < 0) {
 	    gpsd_report(LOG_ERROR, "read-only device open failed: %s\n", strerror(errno));
 	    return -1;
@@ -248,16 +265,18 @@ int gpsd_open(struct gps_device_t *session)
     return session->gpsdata.gps_fd;
 }
 
-bool gpsd_write(struct gps_device_t *session, void const *buf, size_t len)
+ssize_t gpsd_write(struct gps_device_t *session, void const *buf, size_t len)
 {
      ssize_t status;
      bool ok;
+     if (session->context->readonly)
+	return 0;
      status = write(session->gpsdata.gps_fd, buf, len);	
      ok = (status == (ssize_t)len);
      (void)tcdrain(session->gpsdata.gps_fd);
      /* no test here now, always print as hex */
      gpsd_report(LOG_IO, "=> GPS: %s%s\n", gpsd_hexdump(buf, len), ok?"":" FAILED");
-     return ok;
+     return status;
 }
 
 /*
@@ -276,7 +295,7 @@ bool gpsd_next_hunt_setting(struct gps_device_t *session)
     static unsigned int rates[] = {FIXED_PORT_SPEED};
 #else /* FIXED_PORT_SPEED not defined */
     /* every rate we're likely to see on a GPS */
-    static unsigned int rates[] = {0, 4800, 9600, 19200, 38400, 57600};
+    static unsigned int rates[] = {0, 4800, 9600, 19200, 38400, 57600, 115200};
 #endif /* FIXED_PORT_SPEED defined */
 
     if (session->packet.retry_counter++ >= SNIFF_RETRIES) {
@@ -288,7 +307,8 @@ bool gpsd_next_hunt_setting(struct gps_device_t *session)
 	}
 	gpsd_set_speed(session, 
 		       rates[session->baudindex],
-		       'N', session->gpsdata.stopbits);
+		       (unsigned char)session->gpsdata.parity, 
+		       session->gpsdata.stopbits);
     }
 
     return true;	/* keep hunting */
@@ -305,6 +325,15 @@ void gpsd_assert_sync(struct gps_device_t *session)
      */
     if (session->saved_baud == -1)
 	session->saved_baud = (int)cfgetispeed(&session->ttyset);
+
+#ifdef NTPSHM_ENABLE
+    /*
+     * Now is the right time to grab the shared memory segment(s)
+     * to communicate the navigation message derived and (possibly)
+     * 1pps derived time data to ntpd.
+     */
+    ntpd_link_activate(session);
+#endif /* NTPSHM_ENABLE */
 }
 
 void gpsd_close(struct gps_device_t *session)
