@@ -1,4 +1,4 @@
-/* $Id: tsip.c 4030 2006-11-30 07:29:25Z esr $ */
+/* $Id: tsip.c 4651 2008-01-01 14:53:16Z esr $ */
 /*
  * Handle the Trimble TSIP packet format
  * by Rob Janssen, PE1CHL.
@@ -199,7 +199,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f2 = getf(buf,6);			/* leap seconds */
 	if (f1 >= 0.0 && f2 > 10.0) {
 	    session->driver.tsip.gps_week = s1;
-	    session->context->leap_seconds = (int)roundf(f2);
+	    session->context->leap_seconds = (int)round(f2);
 	    session->context->valid |= LEAP_SECOND_VALID;
 
 	    session->gpsdata.sentence_time = gpstime_to_unix((int)s1, f1) - f2;
@@ -272,7 +272,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		f1 = 0.0;
 	    for (j = 0; j < TSIP_CHANNELS; j++)
 		if (session->gpsdata.PRN[j] == (int)u1) {
-		    session->gpsdata.ss[j] = (int)roundf(f1);
+		    session->gpsdata.ss[j] = (int)round(f1);
 		    break;
 		}
 	    (void)snprintf(buf2+strlen(buf2), sizeof(buf2)-strlen(buf2),
@@ -298,7 +298,11 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		gpstime_to_unix((int)session->driver.tsip.gps_week, f2) - session->context->leap_seconds;
 	    mask |= TIME_SET;
 	}
-	gpsd_report(LOG_PROG, "GPS LLA %f %f %f\n",session->gpsdata.fix.latitude,session->gpsdata.fix.longitude,session->gpsdata.fix.altitude);
+	gpsd_report(LOG_PROG, "GPS LLA %f %f %f %f\n",
+		    session->gpsdata.fix.time,
+		    session->gpsdata.fix.latitude,
+		    session->gpsdata.fix.longitude,
+		    session->gpsdata.fix.altitude);
 	mask |= LATLON_SET | ALTITUDE_SET | CYCLE_START_SET;
 	break;
     case 0x4b:		/* Machine/Code ID and Additional Status */
@@ -404,7 +408,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	if (i < TSIP_CHANNELS) {
 	    if (d1 >= 0.0) {
 		session->gpsdata.PRN[i] = (int)u1;
-		session->gpsdata.ss[i] = (int)roundf(f1);
+		session->gpsdata.ss[i] = (int)round(f1);
 		session->gpsdata.elevation[i] = (int)round(d1);
 		session->gpsdata.azimuth[i] = (int)round(d2);
 	    } else {
@@ -513,7 +517,11 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		gpstime_to_unix((int)session->driver.tsip.gps_week, f1) - session->context->leap_seconds;
 	    mask |= TIME_SET;
 	}
-	gpsd_report(LOG_INF, "GPS DP LLA %f %f %f\n",session->gpsdata.fix.latitude,session->gpsdata.fix.longitude,session->gpsdata.fix.altitude);
+	gpsd_report(LOG_INF, "GPS DP LLA %f %f %f %f\n",
+		    session->gpsdata.fix.time,
+		    session->gpsdata.fix.latitude,
+		    session->gpsdata.fix.longitude,
+		    session->gpsdata.fix.altitude);
 	mask |= LATLON_SET | ALTITUDE_SET | CYCLE_START_SET;
 	break;
     case 0x8f:		/* Super Packet.  Well...  */
